@@ -5,16 +5,16 @@ accessible.
 @author: Brett Clippingdale
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
-import Base, Constants
+import Base
 from LSRInterfaces import *
 
 class ChildrenChange(Base.AccessEngineEvent):
@@ -22,8 +22,8 @@ class ChildrenChange(Base.AccessEngineEvent):
   Event that fires on children changes.
   
   This class registers its name and whether it should be monitored by default in
-  an L{AEMonitor.EventMonitor} using the L{Constants.registerEventType} function
-  when this module is first imported. The L{AEMonitor.EventMonitor} can use this
+  an L{AEMonitor} using the L{Base.registerEventType} function
+  when this module is first imported. The L{AEMonitor} can use this
   information to build its menus.
 
   @ivar por: The L{POR} of the parent
@@ -33,7 +33,7 @@ class ChildrenChange(Base.AccessEngineEvent):
   @ivar child_por: The L{POR} of added/removed child
   @type child_por: L{POR}
   '''
-  Constants.registerEventType('ChildrenChange', False)
+  Base.registerEventType('ChildrenChange', False)
   def __init__(self, por, added, child_por, **kwargs):
     '''
     Stores the L{POR}, event name, first and last children (if any) associated
