@@ -3,25 +3,26 @@ Defines an L{AEEvent} indicating a gesture has been performed on an L{AEInput}
 device.
 
 @author: Peter Parente
+@author: Scott Haeger
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import AEInput
-import Base, Constants
+import Base
 
 class InputGesture(Base.AccessEngineEvent):
   '''  
   Event that fires when some L{AEInput.Gesture} is detected on an L{AEInput}
-  device. Always defaults to the L{Constants.LAYER_FOCUS}.
+  device. Always defaults to the focused layer.
   
   This class registers its name and whether it should be monitored by default in
-  an L{AEMonitor} using the L{Constants.registerEventType} function
+  an L{AEMonitor} using the L{Base.registerEventType} function
   when this module is first imported. The L{AEMonitor} can use this
   information to build its menus.
   
@@ -29,8 +30,10 @@ class InputGesture(Base.AccessEngineEvent):
   @type gesture: L{AEInput.Gesture}
   @ivar timestamp: Time at which the gesture was received on the device
   @type timestamp: float
+  @ivar kwargs: Additional arguments to be passed to a L{Task.InputTask}
+  @type kwargs: dictionary
   '''
-  Constants.registerEventType('InputGesture', False)
+  Base.registerEventType('InputGesture', False)
   def __init__(self, gesture, timestamp, **kwargs):
     '''
     Calls the base class (which sets the event priority) and then stores the 
@@ -40,10 +43,13 @@ class InputGesture(Base.AccessEngineEvent):
     @type gesture: L{AEInput.Gesture}
     @param timestamp: Time at which the gesture was received on the device
     @type timestamp: float
+    @ivar kwargs: Additional arguments to be passed to a L{Task.InputTask}
+    @type kwargs: dictionary
     '''
     Base.AccessEngineEvent.__init__(self, focused=True, **kwargs)
     self.gesture = gesture
     self.timestamp = timestamp
+    self.kwargs = kwargs
     
   def __str__(self):
     '''
@@ -97,9 +103,17 @@ class InputGesture(Base.AccessEngineEvent):
   
   def getDataForTask(self):
     '''
-    Provides no information to the L{Task} about this event.
+    Fetches data out of this L{InputGesture} for use by a
+    L{Task.InputTask}.
     
-    @return: Empty dict
+    @return: Dictionary of parameters to be passed to a L{Task.InputTask}
+      as follows:
+        - timestamp: The time when the input gesture occurred. The scale and
+          absolute zero is device dependent.
+        - any additional data in the L{kwargs} instance variable        
     @rtype: dictionary
     '''
-    return {}
\ No newline at end of file
+    d = {}
+    d.update(self.kwargs)
+    d['timestamp'] = self.timestamp
+    return d
