@@ -7,7 +7,7 @@ descendants.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
@@ -84,7 +84,7 @@ class ContainerAccInfoAdapter(DefaultAccInfoAdapter):
     Gets the accessible role name of the subject if item offset is None else 
     the role name of the given item. The name is localized.
     
-    @return: Accessible name of requested object
+    @return: Accessible role name of requested object
     @rtype: string
     @raise LookupError: When the subject accessible or the child is dead
     @raise IndexError: When the item offset is outside the bounds of the 
@@ -100,6 +100,26 @@ class ContainerAccInfoAdapter(DefaultAccInfoAdapter):
       return unicode(c.getLocalizedRoleName(), 'utf-8')
     except AttributeError:
       raise IndexError
+    
+  @pyLinAcc.errorToLookupError
+  def getAccRole(self):
+    '''
+    Gets the accessible role of the subject if item offset is None else 
+    the role name of the given item.
+    
+    @return: Accessible role of requested object
+    @rtype: string
+    @raise LookupError: When the subject accessible or the child is dead
+    @raise IndexError: When the item offset is outside the bounds of the 
+      number of children of the subject accessible
+    '''
+    if self.item_offset is None:
+      return super(ContainerAccInfoAdapter, self).getAccRole()
+    try:
+      c = self.accessible.getChildAtIndex(self.item_offset)
+      return c.getRoleName()
+    except AttributeError:
+      raise IndexError
   
   @pyLinAcc.errorToLookupError
   def getAccName(self):
