@@ -7,7 +7,7 @@ interface on L{pyLinAcc.Accessibility.Accessible} objects.
 @author: Brett Clippingdale
 @author: Eirikur Hallgrimsson
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
@@ -491,6 +491,18 @@ class DefaultAccInfoAdapter(PORAdapter):
     @raise LookupError: When the accessible object is dead
     '''
     return unicode(self.accessible.getLocalizedRoleName(), 'utf-8')
+  
+  @pyLinAcc.errorToLookupError
+  def getAccRole(self):
+    '''
+    Gets the accessible role of the subject. Ignores the given item offset 
+    because the default adapter assumes there are no items.
+    
+    @return: Unlocalized role
+    @rtype: string
+    @raise LookupError: When the accessible object is dead
+    '''
+    return self.accessible.getRoleName()
     
   @pyLinAcc.errorToLookupError
   def getAccName(self):
@@ -710,6 +722,20 @@ class DefaultAccInfoAdapter(PORAdapter):
     '''
     return role == self.accessible.getRoleName()
 
+  @pyLinAcc.errorToLookupError
+  def hasAccOneRole(self, *roles):
+    '''
+    Gets if the subject has any one of the given roles. The given role strings
+    are compared to the unlocalized string role of the accessible.
+    
+    @param roles: Names of the roles (e.g. 'terminal', 'glass pane')
+    @type roles: list of string
+    @return: Does the accessible have any one of the given roles?
+    @rtype: boolean
+    '''
+    return self.accessible.getRoleName() in roles
+    
+
   @pyLinAcc.errorToLookupError
   def getAccActionNames(self):
     '''
