@@ -5,8 +5,9 @@ interface.
 @author: Pete Brunet
 @author: Peter Parente
 @author: Brett Clippingdale
+@author: Scott Haeger
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
@@ -21,7 +22,7 @@ from DefaultInfo import DefaultAccInfoAdapter
 from DefaultNav import *
 from DefaultEventHandler import DefaultEventHandlerAdapter
 from pyLinAcc import Constants, Interfaces
-import pyLinAcc
+import pyLinAcc, AEConstants
 from i18n import _
 import time
 
@@ -122,41 +123,37 @@ class TextAccInfoAdapter(DefaultAccInfoAdapter):
   @pyLinAcc.errorToLookupError
   def getAccVisualExtents(self):
     '''
-    Gets the visual width and height of a line or the entire text area if
+    Gets the visual width and height of a character or the entire text area if
     item offset is None.
 
     @return: Width and height extents
     @rtype: 2-tuple of integer
     @raise LookupError: When the accessible object is dead
     '''
-    it = Interfaces.IText(self.accessible)
     if self.item_offset is None:
-      return super(TextAccInfoAdapter, self).getVisualExtents()
-    line, s, e = text.getTextAtOffset(self.item_offset, 
-                                      Constants.TEXT_BOUNDARY_LINE_START)
-    xs, ys, ws, hs = it.getCharacterExtents(s, Constants.DESKTOP_COORDS)
-    xe, ye, we, he = it.getCharacterExtents(e, Constants.DESKTOP_COORDS)
-    return (xe-xs), (ye-ys)
+      return super(TextAccInfoAdapter, self).getAccVisualExtents()
+    off = self.item_offset + self.char_offset
+    text = Interfaces.IText(self.accessible)
+    x, y, w, h = text.getCharacterExtents(off, Constants.DESKTOP_COORDS)
+    return w, h
 
   @pyLinAcc.errorToLookupError
   def getAccVisualPoint(self):
     '''
-    Gets the center of a line or the entire text area if item offset is None.
+    Gets the center of a character or the entire text area if item offset is
+    None.
 
     @return: x,y coordinates
     @rtype: 2-tuple of integer
     @raise LookupError: When the accessible object is dead
     '''
     if self.item_offset is None:
-      return super(TextAccInfoAdapter, self).getVisualPoint()
+      return super(TextAccInfoAdapter, self).getAccVisualPoint()
     # compute the offset in the line
     off = self.item_offset + self.char_offset
-    acc = self.accessible
-    text = Interfaces.IText(acc)
-    line, s, e = text.getTextAtOffset(off, Constants.TEXT_BOUNDARY_LINE_START)
-    x, y, w, h = text.getCharacterExtents(self.item_offset + self.char_offset,
-                                          Constants.DESKTOP_COORDS)
-    return x, y+h/2
+    text = Interfaces.IText(self.accessible)
+    x, y, w, h = text.getCharacterExtents(off, Constants.DESKTOP_COORDS)
+    return x+w/2, y+h/2
 
 class TextNavAdapter(DefaultNavAdapter):
   '''
@@ -186,7 +183,7 @@ class TextNavAdapter(DefaultNavAdapter):
   @pyLinAcc.errorToLookupError
   def _getVisibleItemExtents(self, only_visible):
     '''
-    Gets the item offsets of the first and last visible items in a text control.
+    Gets the item offsets of the first and last items in a text control.
     
     @param only_visible: Only consider the first and last characters visible in
       the text box (True) or the absolute first and last characters (False)?
@@ -199,17 +196,16 @@ class TextNavAdapter(DefaultNavAdapter):
     comp = Interfaces.IComponent(acc)
     text = Interfaces.IText(acc)
     if only_visible:
-      e = comp.getExtents(Constants.WINDOW_COORDS)
+      e = comp.getExtents(Constants.DESKTOP_COORDS)
       # get the first item
-      x, y = e.x+FUDGE_PX, e.y+FUDGE_PX
-      first = text.getOffsetAtPoint(x, y, Constants.WINDOW_COORDS)
+      first = text.getOffsetAtPoint(e.x, e.y, Constants.DESKTOP_COORDS)
       # get the last item
-      x, y = e.x+e.width-FUDGE_PX, e.y+e.height-FUDGE_PX
-      last = text.getOffsetAtPoint(x, y, Constants.WINDOW_COORDS)
+      last = text.getOffsetAtPoint(e.x+e.width, e.y+e.height, \
+                                   Constants.DESKTOP_COORDS)
     else:
       first = 0
       last = text.characterCount-1
-    if first == last:
+    if first == last or first == -1 or last == -1:
       # nit: protection against bad bounds reporting
       first = 0
       last = text.characterCount-1
@@ -221,8 +217,6 @@ class TextNavAdapter(DefaultNavAdapter):
     Gets the next item relative to the one indicated by the L{POR}
     providing this interface.
 
-    Currently, this assumes only_visible is True.
-    
     @param only_visible: True when Item in the returned L{POR} must be visible
     @type only_visible: boolean
     @return: Point of regard for the beginning of the next item in the same 
@@ -485,14 +479,14 @@ class SimpleTextEventHandlerAdapter(DefaultEventHandlerAdapter):
     
     if n == 0:
       # fire an event with no text if there is no selection
-      return (SelectorChange(por, u'', SelectorChange.CHANGE_TEXT_SELECT,
+      return (SelectorChange(por, u'', AEConstants.EVENT_CHANGE_TEXT_SELECT,
                              **kwargs),)
     else:
       # we don't know which selection changed, so we'll just assume the first
       # one for the time being; multi-select editors are rare
       sel = text.getText(*text.getSelection(0))
       return (SelectorChange(por, unicode(sel, 'utf-8'), 
-                             SelectorChange.CHANGE_TEXT_SELECT, **kwargs),)
+                             AEConstants.EVENT_CHANGE_TEXT_SELECT, **kwargs),)
 
 class TextEventHandlerAdapter(SimpleTextEventHandlerAdapter):
   '''  
