@@ -7,7 +7,7 @@ contents of the profile views using information from the L{UIRegistrar}.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
@@ -23,10 +23,12 @@ from pyLinAcc import Atk
 import AEChooser, AEState
 from i18n import _, DOMAIN
 from GTKUIEView import *
-import weakref
+import weakref, logging
 
 __uie__ = dict(kind='chooser')
 
+log = logging.getLogger('SettingsChooser')
+
 class WidgetFactory(object):
   '''
   Widget factory for creating gtk views for L{AEState.Setting} objects. Has
@@ -34,101 +36,183 @@ class WidgetFactory(object):
   for internally handling widget events and updating their correspoding 
   settings.
   
-  @ivar chooser: Chooser that is requesting the creation of widgets
-  @type chooser: L{AEChooser}
-  '''
-  def __init__(self, chooser):
-    '''
-    Stores a reference to the chooser creating this factory.
+  @ivar last_setting: Last setting object to change. Used to prevent cyclic
+    notification between widgets and setting objects.
+  @type last_setting: L{AEState.Setting}
+  ''' 
+  def __init__(self):
+    self.last_setting = None
     
-    @param chooser: Chooser that is requesting the creation of widgets
-    @type chooser: L{AEChooser}
+  def _isCycle(self, setting):
     '''
-    self.chooser = weakref.proxy(chooser)
+    Detects duplicate notifications of setting changes. This can be caused by
+    a pygtk widget changing, updating a setting, and then getting renotified
+    by the setting about the change. It can also occur when the setting 
+    changes, updates the widget, and the widget tries to update the setting
+    again.
     
-  def _signalSetting(self, setting):
-    self.chooser._signalSetting(setting.context, setting.variable, 
-                                setting.value)
+    @param setting: Setting which is about to change or has changed value
+    @type setting: L{AEState.Setting}
+    '''
+    if self.last_setting is setting:
+      # don't notify ourselves again
+      self.last_setting = None
+      return True
+    self.last_setting = setting
+    return False
   
-  def _updateBool(self, widget, setting):
+  def _boolWidgetChange(self, widget, setting):
+    if self._isCycle(setting): return
     setting.value = widget.get_active()
-    self._signalSetting(setting)
     
-  def _updateNumeric(self, adj, setting):
-    setting.value = adj.get_value()
-    self._signalSetting(setting)
+  def _boolSettingChange(self, state, setting, widget):
+    if self._isCycle(setting): return
+    widget.set_active(setting.value)
     
-  def _updateFilename(self, widget, setting):
+  def _numericWidgetChange(self, adj, setting):
+    if self._isCycle(setting): return
+    setting.getView().value = adj.get_value()
+    
+  def _numericSettingChange(self, state, setting, widget):
+    if self._isCycle(setting): return
+    widget.set_value(setting.getView().value)
+    
+  def _filenameWidgetChange(self, widget, setting):
+    if self._isCycle(setting): return
     fn = widget.get_filename()
     if fn is not None:
       setting.value = fn
-    self._signalSetting(setting)
     
-  def _updateString(self, widget, event, setting):
-    setting.value = widget.get_text()
-    self._signalSetting(setting)
+  def _filenameSettingChange(self, state, setting, widget):
+    if self._isCycle(setting): return
+    widget.set_filename(value)
 
-  def _updateChoice(self, widget, setting):
+  def _stringWidgetChange(self, widget, event, setting):
+    if self._isCycle(setting): return
+    setting.value = widget.get_text()
+    
+  def _stringSettingChange(self, state, setting, widget):
+    if self._isCycle(setting): return
+    widget.set_text(value)
+
+  def _choiceWidgetChange(self, widget, setting):
+    if self._isCycle(setting): return
     index = widget.get_active()
     setting.value = setting.values[index]
-    self._signalSetting(setting)
+    
+  def _choiceSettingChange(self, state, setting, widget):
+    if self._isCycle(setting): return
+    widget.set_active(setting.values.index(setting.value))
+    
+  def _colorWidgetChange(self, widget, setting):
+    if self._isCycle(setting): return
+    color = widget.get_color()
+    if widget.get_use_alpha():
+      alpha = widget.get_alpha()
+      setting.value = (color.red, color.green, color.blue, alpha)
+    else:
+      setting.value = (color.red, color.green, color.blue)
+      
+  def _colorSettingChange(self, state, setting, widget):
+    if self._isCycle(setting): return
+    widget.set_color(gtk.gdk.Color(red=value[0], green=value[1],
+                                   blue=value[2]))
+    if len(value) > 3:
+      widget.set_alpha(value[3])
   
   def create(self, setting):
     widget = None
     if isinstance(setting, AEState.Group):
       # another group
-      widget = gtk.Frame(setting.name)
-      widget.set_shadow_type(gtk.SHADOW_ETCHED_IN)
+      widget = gtk.Frame()
+      widget.set_shadow_type(gtk.SHADOW_NONE)
+      label = gtk.Label()
+      label.set_markup('<b>%s</b>' % setting.name)
+      widget.set_label_widget(label)
       return widget, None
     elif isinstance(setting, AEState.BoolSetting):
       # boolean
-      widget = gtk.CheckButton()
+      widget = gtk.CheckButton(setting.label)
       widget.set_active(setting.value)
-      widget.connect('toggled', self._updateBool, setting)
-      # set the accessible name
-      Atk.setNameDesc(widget, name=setting.label)
+      id = widget.connect('toggled', 
+                          AEState.Proxy(self._boolWidgetChange), setting)
+      # register for setting value changes
+      setting.addObserver(self._boolSettingChange, widget)
+      return widget, None
     elif isinstance(setting, AEState.FilenameSetting):
       # filename
       widget = gtk.FileChooserButton(setting.label)
       widget.set_filename(setting.value)
-      widget.connect('selection-changed', self._updateFilename, setting)
+      widget.connect('selection-changed', 
+                     AEState.Proxy(self._filenameWidgetChange), setting)
+      # register for setting value changes
+      setting.addObserver(self._filenameSettingChange, widget)
     elif isinstance(setting, AEState.StringSetting):
       # string
       widget = gtk.Entry()
       widget.set_text(setting.value)
-      widget.connect('focus-out-event', self._updateString, setting)
+      widget.connect('focus-out-event', 
+                     AEState.Proxy(self._stringWidgetChange), setting)
+      # register for setting value changes
+      setting.addObserver(self._stringSettingChange, widget)
     elif isinstance(setting, AEState.NumericSetting):
+      # use a view object for a range
+      view = setting.getView()
       # numeric or range
-      if setting.precision == 0:
+      if view.precision == 0:
         step = 1
       else:
-        step = 0.1**setting.precision
-      page = int((setting.max-setting.min)/10.0)
-      adj = gtk.Adjustment(setting.value, setting.min, setting.max, step, page)
-      if isinstance(setting, AEState.RangeSetting):
+        step = 0.1**view.precision
+      page = int((view.max-view.min)/10.0)
+      adj = gtk.Adjustment(view.value, view.min, view.max, step, page)
+      if isinstance(setting, (AEState.RangeSetting, AEState.RelRangeSetting)):
         widget = gtk.HScale(adj)
       else:
         widget = gtk.SpinButton(adj)
-      widget.set_digits(setting.precision)
-      widget.set_value(setting.value)
-      adj.connect('value-changed', self._updateNumeric, setting)
+      widget.set_digits(view.precision)
+      widget.set_value(view.value)
+      adj.connect('value-changed', AEState.Proxy(self._numericWidgetChange)
+                  , setting)
+      # register for setting value changes
+      setting.addObserver(self._numericSettingChange, widget)
     elif isinstance(setting, AEState.EnumSetting):
       # enum
       widget = gtk.combo_box_new_text()
       map(widget.append_text, setting.labels)
       widget.set_active(setting.values.index(setting.value))
-      widget.connect('changed', self._updateChoice, setting)
+      widget.connect('changed', 
+                     AEState.Proxy(self._choiceWidgetChange), setting)
+      # register for setting value changes
+      setting.addObserver(self._choiceSettingChange, widget)
     elif isinstance(setting, AEState.ChoiceSetting):
       # choice
       widget = gtk.combo_box_new_text()
       map(widget.append_text, setting.values)
       widget.set_active(setting.values.index(setting.value))
-      widget.connect('changed', self._updateChoice, setting)
+      widget.connect('changed', 
+                     AEState.Proxy(self._choiceWidgetChange), setting)
+      # register for setting value changes
+      setting.addObserver(self._choiceSettingChange, widget)
+    elif isinstance(setting, AEState.ColorSetting):
+      # color
+      widget = gtk.ColorButton(gtk.gdk.Color(red=setting.value[0],
+                                             green=setting.value[1],
+                                             blue=setting.value[2]))
+      if len(setting.value) > 3:
+        widget.set_use_alpha(True)
+        widget.set_alpha(setting.value[3])
+      else:
+        widget.set_use_alpha(False)
+      widget.connect('color-set', 
+                     AEState.Proxy(self._colorWidgetChange), setting)
+      # register for setting value changes
+      setting.addObserver(self._colorSettingChange, widget)
     else:
       raise NotImplementedError
     # create a label
     label = gtk.Label(setting.label)
-    label.set_alignment(1.0, 0.5)   
+    label.set_alignment(0, 0.5)
+    
     return widget, label
 
 class SettingsChooser(AEChooser.AEChooser): 
@@ -141,11 +225,14 @@ class SettingsChooser(AEChooser.AEChooser):
   @ivar dialog: Dialog widget for configuring LSR settings
   @type dialog: gtk.Dialog
   @ivar system: LSR system wide settings
-  @type system: L{AEState.Setting.Group}
-  @ivar perks: Per L{Perk} settings keyed by Perk name
-  @type perks: dictionary of string : L{AEState.Setting.Group}
+  @type system: L{AEState}
+  @ivar perks: Per L{Perk} settings keyed by human readable Perk name
+  @type perks: dictionary of string : L{AEState}
+  @ivar devices: Per L{AEOutput} device settings keyed by human readable device
+    name
+  @type devices: dictionary of string : L{AEState}
   @ivar installed: All UIE class names, human readable names, and 
-  descriptions installed keyed by their type
+    descriptions installed keyed by their type
   @type installed: dictionary of string : 3-tuple of string
   @ivar associated: All UIE class names associated with this profile
   @type associated: list of string
@@ -207,11 +294,11 @@ class SettingsChooser(AEChooser.AEChooser):
     Creates and shows the settings dialog and its components
 
     @param system: LSR system wide settings
-    @type system: L{AEState.Setting.Group}
+    @type system: L{AEState}
     @param perks: Per L{Perk} settings keyed by Perk name
-    @type perks: dictionary of string : L{AEState.Setting.Group}
+    @type perks: dictionary of string : L{AEState}
     @param devices: Per L{AEOutput}/L{AEInput} settings keyed by device name
-    @type devices: dictionary of string : L{AEState.Setting.Group}
+    @type devices: dictionary of string : L{AEState}
     @param installed: All UIE class names, human readable names, and 
       descriptions installed keyed by their type
     @type installed: dictionary of string : list of 3-tuple of string
@@ -225,8 +312,8 @@ class SettingsChooser(AEChooser.AEChooser):
     @type profile: string
     '''
     # create a widget factory
-    self.factory = WidgetFactory(self)
-    
+    self.factory = WidgetFactory()
+
     # store group objects
     self.system = system
     self.perks = perks
@@ -347,7 +434,7 @@ class SettingsChooser(AEChooser.AEChooser):
     '''
     Populates the system settings profile panel.
     '''
-    layout = self._populateSection(self.system, self.system_vp)
+    layout = self._populateSection(self.system.getGroups(), self.system_vp)
     self.system_vp.add(layout)
     self.system_vp.show_all()
     
@@ -380,10 +467,17 @@ class SettingsChooser(AEChooser.AEChooser):
     tvc = gtk.TreeViewColumn('', crt, text=0)
     self.perk_tv.append_column(tvc)
     
-    # populate the list of perks with settings
+    # populate the list of perks
     if len(self.perks) > 0:
-      for perk in self.perks.keys():
-        model.append([perk])
+      keys = self.perks.keys()
+      keys.sort()
+      for name in keys:
+        try:
+          self.perks[name].getGroups()
+        except NotImplementedError:
+          # ignore states that have no configurable settings
+          continue
+        model.append([name])
       # set the first one as selected to cause an update of the panel
       self.perk_tv.set_cursor((0,))
       
@@ -405,8 +499,15 @@ class SettingsChooser(AEChooser.AEChooser):
     
     # populate the list of perks with settings
     if len(self.devices) > 0:
-      for device in self.devices.keys():
-        model.append([device])
+      keys = self.devices.keys()
+      keys.sort()
+      for name in keys:
+        try:
+          self.devices[name].getGroups()
+        except NotImplementedError:
+          # ignore states that have no configurable settings
+          continue
+        model.append([name])
       # set the first one as selected to cause an update of the panel
       self.device_tv.set_cursor((0,))
 
@@ -456,11 +557,19 @@ class SettingsChooser(AEChooser.AEChooser):
     label.set_markup(self.FRAME_MARKUP % name)
     # populate the subsection
     if self.perk_tv == tv:
-      layout = self._populateSection(self.perks[name], viewport)
-      viewport.add(layout)
+      group = self.perks[name].getGroups()
+      try:
+        layout = self._populateSection(group, viewport)
+        viewport.add(layout)
+      except AssertionError:
+        log.warn('empty perk settings group')
     elif self.device_tv == tv:
-      layout = self._populateSection(self.devices[name], viewport)
-      viewport.add(layout)
+      group = self.devices[name].getGroups()
+      try:
+        layout = self._populateSection(group, viewport)
+        viewport.add(layout)
+      except AssertionError:
+        log.warn('empty device settings group')
     viewport.show_all()
 
   def _populateSection(self, group, viewport):
@@ -470,20 +579,24 @@ class SettingsChooser(AEChooser.AEChooser):
     should be made. If recursive groups exist, their settings will be properly
     represented by widgets in a widget sub-container.
 
-    @param group: Group of settings linked to actual settings values
+    @param group: Group organizing a subset of settings in the state
     @type group: L{AEState.Setting.Group}
+    @param viewport: Reference to the viewport to populate with widgets
+    @type viewport: gtk.Viewport
+    @raise AssertionError: When there are no settings in the group
     '''
     n = len(group)
+    if n < 1:
+      # no settings in the group
+      raise AssertionError
     # use a n by 2 table to order widgets nicely
     table = gtk.Table(n, 2)
     table.set_row_spacings(5)
     table.set_col_spacings(5)
     # nit: add in reverse order else controls show up backwards in acc. tree
-    group = list(group)
-    group.reverse()
-    for row, setting in enumerate(group):
+    for row, setting in group.iterSettings(reverse=True):
       widget, label = self.factory.create(setting)
-      if label is None:
+      if isinstance(widget, gtk.Frame):
         # if the widget is another group, recurse
         layout = self._populateSection(setting, viewport)
         layout.set_border_width(5)
@@ -495,19 +608,26 @@ class SettingsChooser(AEChooser.AEChooser):
         table.attach(widget, 0, 2, n-row-1, n-row, yoptions=gtk.FILL, 
                      xpadding=5)
       else:
+        if isinstance(widget, gtk.CheckButton):
+          # check buttons have a label already, so fill the row
+          table.attach(widget, 0, 2, n-row-1, n-row, yoptions=gtk.FILL)
+        else:
+          # relate the label to the widget
+          Atk.setRelation(widget, atk.RELATION_LABELLED_BY, label)
+          Atk.setRelation(label, atk.RELATION_LABEL_FOR, widget)
+          # attach to the table
+          # note: add widget on the right first then label on the left,
+          # else gail screws up the order in the acc. hierarchy
+          table.attach(widget, 1, 2, n-row-1, n-row, yoptions=gtk.FILL)
+          table.attach(label, 0, 1, n-row-1, n-row, gtk.FILL, gtk.FILL, 5, 0)
+          # watch for focus on the widget so we can scroll
         if setting.description:
-          # set description as tooltip and accessible description
+          # set description as a tooltip on the widget itself; does not work
+          # with all widgets (e.g. combobox)
           self.tips.set_tip(widget, setting.description)
+          # also set description as accessible description on the widget
           Atk.setNameDesc(widget, description=setting.description)
-        # relate the label to the widget
-        Atk.setRelation(widget, atk.RELATION_LABELLED_BY, label)
-        Atk.setRelation(label, atk.RELATION_LABEL_FOR, widget)
-        # attach to the table
-        # nit: add widget on the right first then label on the left, else gail
-        # screws up the order in the acc. hierarchy
-        table.attach(widget, 1, 2, n-row-1, n-row, yoptions=gtk.FILL)
-        table.attach(label, 0, 1, n-row-1, n-row, gtk.FILL, gtk.FILL, 5, 0)
-        # watch for focus on the widget so we can scroll
+        # watch for focus so we can scroll the widget into view
         widget.connect('focus', self._onScrollToFocus, viewport)
     return table
   
@@ -566,12 +686,13 @@ class SettingsChooser(AEChooser.AEChooser):
     Closes the chooser, preventing further chooser interaction with the user.
     '''
     self.tips = None
+    self.factory = None
     gtk.glade.set_custom_handler(lambda x: None)
     self.dialog.destroy()
 
   def getName(self):
     '''
-    Gets the name of the chooser.    
+    Gets the name of the chooser.
     
     @return: Human readable name of the chooser
     @rtype: string
@@ -583,14 +704,7 @@ class SettingsChooser(AEChooser.AEChooser):
     Updates the list of devices according to those that are now loaded.
     
     @param devices: Per L{AEOutput}/L{AEInput} settings keyed by device name
-    @type devices: dictionary of string : L{AEState.Setting.Group}
+    @type devices: dictionary of string : L{AEState}
     '''
     self.devices = devices
     self._createDeviceSettingsView()
-  
-  def updateSetting(self, state, name, value):
-    '''
-    @todo: PP: need to get a reference to the panel showing the given state,
-      and the widget representing the named setting
-    '''
-    print 'chooser updating', state, name, value
