@@ -1,6 +1,6 @@
 '''
-Defines a class representing a standard L{Keyboard} input device and action code
-constants used to register filters on that device.
+Defines a class representing a standard L{Keyboard} input device and action 
+code constants used to register filters on that device.
 
 @var MAX_KEY_CHORD: Maximum number of actions that can be in a keyboard gesture
 @type MAX_KEY_CHORD: integer
@@ -9,27 +9,28 @@ constants used to register filters on that device.
 @type def_keymap: gtk.gdk.Keymap
 
 @author: Peter Parente
+@author: Scott Haeger
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
-import logging
-import AEInput, pyLinAcc
+import AEInput, pyLinAcc, AEConstants
 # need pygtk to map keycodes to keynames
 import pygtk
 pygtk.require('2.0')
 import gtk.gdk as gdk
 from i18n import _
 
-log = logging.getLogger('Keyboard')
 def_keymap = gdk.keymap_get_default()
 MAX_KEY_CHORD = 3
 
+__uie__ = dict(kind='device')
+
 def _keyCodeToKeyName(code):
   '''
   Gets the name of the given key code by using the default key map to lookup
@@ -42,14 +43,16 @@ def _keyCodeToKeyName(code):
   @rtype: string
   '''
   entries = def_keymap.get_entries_for_keycode(code)
-  if len(entries):
+  if entries and len(entries):
     return gdk.keyval_name(entries[0][0])
+  return ''
 
 class Keyboard(AEInput.SystemInput):
-  '''    
-  Represents a standard keyboard device attached to the system. Creates an
-  instance of L{pyLinAcc.Event.Manager} to register for system wide keyboard key
-  press and key release events. Matches key presses against a collection of
+  '''
+  Basic keyboard input device.
+  
+  Has a reference to the L{EventManager} to register for system wide keyboard
+  key press and key release events. Matches key presses against a collection of
   stored L{AEInput.Gesture}s representing key combinations that should be
   filtered before they can reach the window with the input focus. Observers of
   this device are notified about L{AEInput.Gesture}s when one or more press
@@ -87,20 +90,25 @@ class Keyboard(AEInput.SystemInput):
   @type state: callable
   @ivar last_press: Was the last event a press (True) or release (False)?
   @type last_press: boolean
+  @ivar stateful: Last stateful key press
+  @type stateful: integer
   @ivar LOCK_KEYS: Key codes for keys that can be locked in a particular state,
     namely L{AEK_CAPS_LOCK}, L{AEK_SCROLL_LOCK}, and L{AEK_NUM_LOCK}. These
     are constant.
   @type LOCK_KEYS: list of integer
   '''
-  def __init__(self):
+  def init(self):
     '''
     Initializes the empty filters dictionary and sets the event_manager
     reference to None. Creates an empty L{AEInput.Gesture} that will be used to
     buffer incoming keystrokes until the L{AEInput.Gesture} definitely matches
-    or does not match one of the held filters.
+    or does not match one of the held filters. Registers for key press and 
+    release events via L{pyLinAcc}. Sets the filtering mode to handled by
+    default. Also registers for window activate events to account for missing 
+    keys caused by system wide hotkey presses.
     '''
-    AEInput.SystemInput.__init__(self)
     self.filters = {}
+    self.stateful = None
     self.event_manager = pyLinAcc.Event.Manager()
     self.curr_gesture = AEInput.Gesture(self)
     self.state = self._stateInvalid
@@ -109,19 +117,11 @@ class Keyboard(AEInput.SystemInput):
     self.last_press = False
     self.LOCK_KEYS = [self.AEK_CAPS_LOCK, self.AEK_SCROLL_LOCK, 
                       self.AEK_NUM_LOCK]
-
-  def init(self):
-    '''
-    Initializes the L{Keyboard} device by registering for key press and release
-    events via the L{EventManager}. Sets the filtering mode to handled by
-    default. Also registers for focus events to account for missing keys caused
-    by system wide hotkey presses.
-    '''
-    AEInput.SystemInput.init(self)
+    
     self.event_manager.addClient(self._onKeyPressed, 'keyboard:press')
     self.event_manager.addClient(self._onKeyReleased, 'keyboard:release')
     self.event_manager.addClient(self._onForegroundChange, 'window:activate')
-    self.setFilterMode(AEInput.Constants.FILTER_HANDLED)
+    self.setFilterMode(AEConstants.FILTER_HANDLED)
 
   def close(self):
     '''
@@ -132,33 +132,42 @@ class Keyboard(AEInput.SystemInput):
     AEInput.SystemInput.close(self)
     self.event_manager.removeClient(self._onKeyPressed, 'keyboard:press')
     self.event_manager.removeClient(self._onKeyReleased, 'keyboard:release')
-    self.event_manager.removeClient(self._onForegroundChange, 'window:activate')
-    self.setFilterMode(AEInput.Constants.FILTER_NONE)
+    self.event_manager.removeClient(self._onForegroundChange,'window:activate')
+    self.setFilterMode(AEConstants.FILTER_NONE)
+    
+  def getCapabilities(self):
+    '''
+    Gets the list of capabilities provided by this device, namely 'keyboard.'
+    
+    @return: 'keyboard' as the only capability of this device.
+    @rtype: list of string
+    '''
+    return ['keyboard']
 
   def addFilter(self, key_filter):
     '''
     Adds a L{AEInput.GestureList} to the collection of filters. The fire
     L{AEInput.Gesture} in the L{AEInput.GestureList} will be used when the mode
-    is set to L{AEInput.Constants.FILTER_HANDLED} to prevent keys consumed by
-    LSR from reaching the foreground application.
+    is set to filter handled to prevent keys consumed by LSR from reaching the 
+    foreground application.
 
     The L{Keyboard} input device only supports one L{AEInput.Gesture} per
-    filter, not a sequence of L{AEInput.Gesture}s, and only one non-modifier key
-    code per L{AEInput.Gesture} chord. Any L{AEInput.GestureList} breaking these
-    rules will cause this method to raise a ValueError.
+    filter, not a sequence of L{AEInput.Gesture}s, and only one non-modifier
+    key code per L{AEInput.Gesture} chord. Any L{AEInput.GestureList} breaking
+    these rules will cause this method to raise a ValueError.
 
     @param key_filter: Filter to add to the filter list
     @type key_filter: L{AEInput.GestureList}
-    @raise ValueError: When the given L{AEInput.GestureList} does not conform to
-      the rules set forth above
+    @raise ValueError: When the given L{AEInput.GestureList} does not conform 
+      to the rules set forth above
     '''
     # make sure there is only one Gesture in the list
     if key_filter.getNumGestures() != 1:
-      raise ValueError(_('Only one Gesture allowed in a Keyboard filter'))
+      raise ValueError('only one Gesture allowed in a Keyboard filter')
     gesture = key_filter.getGestureAt(0)
     # make sure there is at least one keystroke in the Gesture
     if gesture.getNumActions() < 1:
-      raise ValueError(_('At least one key code required'))
+      raise ValueError('at least one key code required')
     # make sure there is only one non-modifier key code
     count = 0
     for code in gesture:
@@ -167,7 +176,7 @@ class Keyboard(AEInput.SystemInput):
       except KeyError:
         count += 1
     if count > 1:
-      raise ValueError(_('At most one non-modifier key allowed'))
+      raise ValueError('at most one non-modifier key allowed')
     # add the filter to the filters dictionary indexed by the first Gesture
     self.filters[gesture] = None
 
@@ -205,23 +214,7 @@ class Keyboard(AEInput.SystemInput):
     @return: Localized device name
     @rtype: string
     '''
-    return _('Keyboard 1.0')
-
-  def sortGesture(self, gesture):
-    '''
-    Sorts the actions in the given L{AEInput.Gesture} and returns them as a list
-    of integers. The sort is done on a copy so the action codes held by the
-    provided L{AEInput.Gesture} are not touched.
-
-    @param gesture: L{AEInput.Gesture} to sort
-    @type gesture: L{AEInput.Gesture}
-    @return: Sorted list of action codes that may be wrapped in a new
-        L{AEInput.Gesture}
-    @rtype: list of integer
-    '''
-    codes = gesture.getActionCodes()
-    codes.sort()
-    return codes
+    return _('Keyboard')
 
   def resetState(self):
     '''
@@ -259,7 +252,7 @@ class Keyboard(AEInput.SystemInput):
         return False
     return True #gesture.getNumActions() != 0
 
-  def _sendKeys(self, gesture, press):
+  def _sendGesture(self, gesture, press):
     '''    
     Synthesizes key strokes for the key codes in the given L{AEInput.Gesture}.
     The presses are noted in L{synth_press} and L{synth_release} so that they
@@ -280,6 +273,23 @@ class Keyboard(AEInput.SystemInput):
         self.synth_press.append(code)
       else:
         self.synth_release.append(code)
+        
+  def _sendKey(self, code, press):
+    '''
+    Synthesizes a single key stroke. The presses are noted in L{synth_press} 
+    and L{synth_release} so that they can be ignored when they are seen in 
+    L{_onKeyPressed} or L{_onKeyReleased} to avoid duplicate processing.
+    
+    @param code: Keycode of the key to generate
+    @type code: integer
+    @param press: Synthesize key press (True) or releas (False)?
+    @type press: boolean
+    '''
+    self.event_manager.generateKeyEvent(code, press)
+    if press:
+      self.synth_press.append(code)
+    else:
+      self.synth_release.append(code)
 
   def _statePossible(self, event, press):
     '''
@@ -315,7 +325,7 @@ class Keyboard(AEInput.SystemInput):
       return self._stateMatch
     else:
       # update the os key state
-      self._sendKeys(self.curr_gesture, True)
+      self._sendGesture(self.curr_gesture, True)
       # move to the invalid state and consume the current key
       event.consume = True
       return self._stateInvalid
@@ -375,11 +385,12 @@ class Keyboard(AEInput.SystemInput):
     '''
     if press:
       if self.filters.has_key(self.curr_gesture):
-        # stay in this state and do nothing, typomatic
+        # stay in this state and eat the key, typomatic
+        event.consume = True
         return self._stateMatch
       else:
         # send all key presses
-        self._sendKeys(self.curr_gesture, True)
+        self._sendGesture(self.curr_gesture, True)
         # move to the invalid state and eat the key
         event.consume = True
         return self._stateInvalid
@@ -442,22 +453,34 @@ class Keyboard(AEInput.SystemInput):
     @param event: Keyboard key press event
     @type event: L{pyLinAcc.Event.Event}
     '''
+    code = event.detail2
     #print 'press:', event.detail2
     if len(self.synth_press) and event.detail2 == self.synth_press[0]:
       # ignore keys we know we synthesized
       self.synth_press.pop(0)
       return
-    if event.detail2 not in self.curr_gesture:
+    if code not in self.curr_gesture:
       # only count a non-typomatic press as a last press
       self.last_press = True
       # don't need to do the following if it was a typomatic press, repeats are
       # ignored when trying to add to a Gesture
       try:
         # try to store the action code in the current gesture
-        self.curr_gesture.addActionCode(event.detail2)
+        self.curr_gesture.addActionCode(code)
       except ValueError:
         # ignore the press if the gesture is full
         return
+
+    # treat stateful keys in a special manner if they're modifiers and another
+    # key has been pressed after one has been held
+    if (self.modifiers.has_key(code) and code in self.LOCK_KEYS):
+      self.stateful = code
+    if (len(self.curr_gesture) > 1 and self.stateful is not None):
+      # build a gesture with the stateful key press only in it
+      g = AEInput.Gesture(self, action_codes=[self.stateful])
+      self._sendGesture(g, True)
+      self.stateful = None
+      
     #print 'state:', self.state
     #print 'curr key:', str(self.curr_gesture)
     self.state = self.state(event, True) or self.state
@@ -493,17 +516,17 @@ class Keyboard(AEInput.SystemInput):
       # ignore keys we know we synthesized
       self.synth_release.pop(0)
       return
-    # treat stateful keys in a special manner if they're modifiers
-    if (self.modifiers.has_key(code) and code in self.LOCK_KEYS):
-      # build a gesture with the stateful key in it
-      g = AEInput.Gesture(self, action_codes=[code])
-      self._sendKeys(g, True)
-      self._sendKeys(g, False)
-    if self.last_press:
+    elif code in self.synth_press:
+      # ignore releases for presses we're waiting to synth
+      # but make sure to synth our own release
+      self._sendKey(code, False)
+      event.consume = True
+      return
+    elif self.last_press:
       # make a copy of the current gesture before we modify it
       gesture = AEInput.Gesture(self, gesture=self.curr_gesture)
-      # notify listeners
-      self._notifyInputListeners(gesture)
+      # notify listeners of the gesture and its timestamp
+      self._notifyInputListeners(gesture, event.any_data[3])
     try:
       self.curr_gesture.removeActionCode(code)
     except ValueError:
@@ -647,4 +670,4 @@ class Keyboard(AEInput.SystemInput):
   AEK_NUMPAD_ENTER = 108
   AEK_THINKPAD_FN = 227
   AEK_THINKPAD_BACK = 234
-  AEK_THINKPAD_FORWARD = 233
\ No newline at end of file
+  AEK_THINKPAD_FORWARD = 233
