@@ -3,80 +3,64 @@ Defines a device that uses the Speech Dispatcher audio server via the speechd
 package. Speech Dispatcher can be obtained from 
 U{http://www.freebsoft.org/speechd}.
 
-This device appears very buggy when using IBM TTS with the version from CVS.
-Probably because Speech Dispatcher is still very early in development.
-
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import time
 import speechd
-import AEOutput
+import AEOutput, AEConstants
 from i18n import _
 
 __uie__ = dict(kind='device')
-
-class SDSpeechStyle(AEOutput.StyleFlyweight):
-  '''
-  Overrides the base L{AEOutput.Style.StyleFlyweight} class to provide style
-  settings relative to the default style for this device.
-  '''
-  ADDITIVE = ('Rate', 'Volume', 'Pitch')
-  Rate = 0
-  Volume = 0
-  Pitch = 0
   
-  def getSettings(self):
-    '''
-    Gets configurable relative settings affecting output from this device for
-    particular types of information.
-    
-    @todo: PP: support in a later version
-    
-    @return: Group of all configurable settings per semantic information type
-    @rtype: L{AEState.Setting.Group}
-    '''
-    raise NotImplementedError
-
-class SDSpeechStyleDefault(AEOutput.AudioStyleDefault):
+class SDSpeechStyle(AEOutput.AudioStyle):
   '''
-  Default style for Speech Dispatcher output.
-  '''
-  MinRate = -100
-  MaxRate = 100
-  MinVolume = -100
-  MaxVolume = 100
-  MinPitch = -100
-  MaxPitch = 100
-  Rate = 60
-  Volume = 180 # about 90%
-  Pitch = 0
-  Voice = 0
+  Style object built dynamically as we discover what capabilities the active
+  gnome-speech driver supports.
+  ''' 
+  def init(self, device):
+    '''
+    Initializes the style object based on the voices reported to be supported
+    by the current device.
+    
+    @param device: Reference to the gnome speech device class
+    @type device: L{SpeechDispatcher}
+    '''
+    voices = device.getVoices()
+    # compute all voice names
+    names = dict(((v, i) for i, v in enumerate(voices)))
+    # make voices into an enum with offset in list as value
+    self.newEnum('Voice', 0, _('Voice'), names, 
+                   _('Name of the active voice'))    
+    self.newRelRange('Pitch', 0, _('Pitch'), -100, 100, 0,
+                     _('Baseline voice pitch'))
+    self.newRelRange('Rate', 60, _('Rate'), -100, 100, 0,
+                     _('Speech rate'))
+    self.newRelPercent('Volume', 60, _('Volume'), -100, 100, 0,
+                       _('Speech volume'))
   
-  def getSettings(self):
+  def getGroups(self):
     '''
     Gets configurable absolute settings affecting all output from this device.
     
     @return: Group of all configurable settings
     @rtype: L{AEState.Setting.Group}
     '''
-    g = self._newGroup()
-    a = g.newGroup(_('Speech'))
-    a.newRange('Pitch', _('Pitch'), self.MinPitch, self.MaxPitch, 0,
-               _('Baseline voice pitch'))
-    a.newRange('Rate', _('Rate'), self.MinRate, self.MaxRate, 0,
-                _('Speech rate'))
-    a.newPercent('Volume', _('Volume'), self.MinVolume, self.MaxVolume, 0,
-                 _('Speech volume as a percentage'))
-    # generate a group for standard word parsing settings
-    self._newWordGroup(g)
+    g = self.newGroup()
+    s = g.newGroup(_('Speech'))
+    s.extend([name for name in ('Pitch', 'Rate', 'Volume')
+              if self.hasSetting(name)])
+    if self.isDefault():
+      # generate a group for standard word parsing settings on the default
+      # object only for now
+      self._newWordGroup(g)
     return g
 
 class SpeechDispatcher(AEOutput.Audio):
@@ -98,6 +82,7 @@ class SpeechDispatcher(AEOutput.Audio):
   USE_THREAD = False
   ALL_VOICES = ['MALE1', 'FEMALE1', 'CHILD_MALE', 'MALE2', 'FEMALE2', 'MALE3', 
                 'FEMALE3', 'CHILD_FEMALE', 'MALE4', 'FEMALE4']
+  STYLE = SDSpeechStyle
   
   def _applyStyle(self, style):
     '''
@@ -108,11 +93,12 @@ class SpeechDispatcher(AEOutput.Audio):
     @param style: Style to apply to future output
     @type style: L{AEOutput.Style}
     '''
+    # @todo: PP: possible problem here if the user switches the backend and
+    # the given voice is no longer supported
     self.speaker.set_voice(self.voices[style.Voice])
-    self.speaker.set_rate(min(max(style.Rate, style.MinRate), style.MaxRate))
-    self.speaker.set_pitch(min(max(style.Pitch,style.MinPitch),style.MaxPitch))
-    self.speaker.set_volume(min(max(style.Volume, style.MinVolume), 
-                                style.MaxVolume))
+    self.speaker.set_rate(style.Rate)
+    self.speaker.set_pitch(style.Pitch)
+    self.speaker.set_volume(style.Volume)
 
   def init(self):
     '''
@@ -121,13 +107,29 @@ class SpeechDispatcher(AEOutput.Audio):
     @raise AEOutput.InitError: When the device can not be initialized
     '''
     try:
-      self.speaker = speechd.Client('LSR')
+      self.speaker = speechd.Client(AEConstants.PROG_NAME)
     except:
       raise AEOutput.InitError
-    self.default_style = SDSpeechStyleDefault()
     self.last_style = None
     self.buffer = []
     self.voices = []
+    
+    # compute which voices are supported
+    for name in self.ALL_VOICES:
+      try:
+        self.speaker.set_voice(name)
+        self.voices.append(name)
+      except AssertionError:
+        pass
+      
+  def getVoices(self):
+    '''
+    Gets the list of all supported voice names.
+    
+    @return: All voice names currently supported
+    @rtype: list
+    '''
+    return self.voices
 
   def _say(self, text):
     '''
@@ -243,27 +245,13 @@ class SpeechDispatcher(AEOutput.Audio):
     @rtype: list of L{AEOutput.Style}
     '''
     styles = []
-    # compute which voices are supported
-    for name in self.ALL_VOICES:
-      try:
-        self.speaker.set_voice(name)
-        self.voices.append(name)
-      except AssertionError:
-        pass
-    # bounds on the number of voices
-    self.default_style.MaxVoice = len(self.voices)-1
-    self.default_style.MinVoice = 0
     for i in range(0, num_groups*num_layers):
       # compute a valid voice number
       v_num = i % len(self.voices)
-      # create a style object
+      # create a style object and init
       s = SDSpeechStyle(self.default_style)
+      s.init(self)
       # store the current voice number
       s.Voice = v_num
-      # store relative values, but set them to zero to make them equivalent
-      # to the default for now
-      s.Rate = 0
-      s.Volume = 0
-      s.Pitch = 0
       styles.append(s)
     return styles
