@@ -7,14 +7,15 @@ accessibles and handling their events.
 @author: Larry Weiss
 @author: Brett Clippingdale
 @author: Eirikur Hallgrimsson
+@author: Scott Haeger
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
 from AccAdapt import Interface
@@ -272,8 +273,8 @@ class IAccessibleInfo(Interface):
    
   def getAccRoleName():
     '''
-    Gets the unicode accessible role name for the object implementing this 
-    interface.
+    Gets the localized unicode accessible role name for the object implementing
+    this interface.
     
     @return: Unicode role name
     @rtype: string
@@ -360,8 +361,8 @@ class IAccessibleInfo(Interface):
     '''
     Gets all of the name:value attribute pairs of an accessible.
     
-    @return: List of name:value pairs in the accessible attributes
-    @rtype: list of string
+    @return: Name:value pairs in the accessible attributes
+    @rtype: dictionary
     @raise LookupError: When the accessible object is dead
     '''
     pass
@@ -463,6 +464,29 @@ class IAccessibleInfo(Interface):
     @raise LookupError: When the accessible object is dead
     '''
     pass
+
+  def getAccVisualExtents():
+    '''
+    Gets the visual width and height of the subject.
+
+    @return: Width and height extents
+    @rtype: 2-tuple of integer
+    @raise LookupError: When the accessible object is dead
+    '''
+    pass
+
+  def getAccVisualPoint():
+    '''
+    Gets the focal point within the subject, where the focal point is defined
+    as the center of the most interesting point of regard. For a simple
+    control, the focal point is the center of the control. For a control with
+    items, the focal point is the center of an item.
+
+    @return: x,y coordinates
+    @rtype: 2-tuple of integer
+    @raise LookupError: When the accessible object is dead
+    '''
+    pass
   
   def getAccFloatValue():
     '''
@@ -479,7 +503,7 @@ class IAccessibleInfo(Interface):
     Gets all of the default text attributes assigned to the subject.
     
     @return: Name:value pairs of the default text attributes
-    @rtype: list of strings
+    @rtype: dictionary
     @raise LookupError: When the accessible object is dead
     '''
     pass
@@ -555,6 +579,17 @@ class IAccessibleInfo(Interface):
     @raise LookupError: When the accessible object is dead
     '''
     pass
+  
+  def getAccRole():
+    '''
+    Gets the unlocalized string role identifier for the object implementing 
+    this interface.
+    
+    @return: Role name
+    @rtype: string
+    @raise LookupError: When the accessible object is dead
+    '''
+    pass
 
   def hasAccRole(role):
     '''
@@ -568,6 +603,17 @@ class IAccessibleInfo(Interface):
     '''
     pass
   
+  def hasOneAccRole(*roles):
+    '''
+    Gets if the subject has any one of the given roles.
+    
+    @param roles: Names of the roles
+    @type roles: list of string
+    @return: Does the accessible have any one of the given roles?
+    @rtype: boolean
+    '''
+    pass
+  
   def hasAccState(state):
     '''
     Gets if the subject has the given state.
@@ -876,13 +922,53 @@ class IAccessibleAction(Interface):
     '''
     pass
   
+  def doMouseClickAtPOR(mouseevent):
+    '''
+    Performs a mouse event on the center of a given point of regard.  If the
+    accessible has a role of text, then set cursor at given location in por
+    
+    @param mouseevent: mouse event to perform
+    @type mouseevent: integer of type EVENT_SYNTHMOUSE*
+    @return: Indicator of whether the acc was clicked or not
+    @raise LookupError: When the accessible object is dead
+    '''
+    pass
+  
 class IEventHandler(Interface):
   '''
   Provides methods for handling accessible events.
   '''
+  def getRawEvents(self, kind):
+    '''
+    Gets a list of raw event names that map to the given kind of L{AEEvent}.
+    
+    @param kind: Indicates the type of L{AEEvent}
+    @type kind: L{AEEvent} class
+    @return: List of raw event names
+    @rtype: list of string
+    @raise KeyError: When no mapping exists for the given event
+    '''
+    pass
+  
+  def getAEViewEvents(self, event, collector, vm):
+    '''
+    Determines if the active view has changed and what events need to be
+    fired in response.
+    
+    @param event: Raw event
+    @type event: L{pyLinAcc.Event.Event}
+    @param collector: Callable object that collects L{AEEvent}s to process. 
+      Accepts N parameters of type L{AEEvent}.
+    @type collector: callable
+    @param vm: Reference to the view manager that has information about the 
+      current view and needs to store a reference to a new view
+    @type vm: L{ViewManager}
+    '''
+    pass
+  
   def getAEEvents(event, collector):
     '''
-    Returns the L{AEEvent}s that need to be posted for this event.
+    Collects the L{AEEvent}s that need to be posted for this event.
     
     @param event: Raw event
     @type event: object
