@@ -3,13 +3,13 @@ Defines tools for assisting L{Perk} developers.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 # import useful modules for Perks
 import Perk, Task
@@ -28,7 +28,11 @@ class DeveloperPerk(Perk.Perk):
     '''
     Registers keys for all L{Task}s in the module.
     '''
-    kbd = self.getInputDevice('Keyboard')
+    # set an audio device as the default output
+    self.setPerkIdealOutput('audio')
+    
+    # register keyboard bindings
+    kbd = self.getInputDevice(None, 'keyboard')
     # set Alt+CapsLock as the modifier
     self.addInputModifiers(kbd, kbd.AEK_ALT_L, kbd.AEK_ALT_R, 
                            kbd.AEK_CAPS_LOCK)
@@ -58,13 +62,13 @@ class Mute(Task.InputTask):
   '''
   def execute(self, **kwargs):
     self.stopAll()
-    mute = self.getMute() 
+    mute = self.getStyleVal('Mute') 
     if mute:
-      self.setMute(False)
+      self.setStyleVal('Mute', False)
       self.sayInfo(text=_('unmuted'))
     else:
       self.sayInfo(text=_('muted'))
-      self.setMute(True)
+      self.setStyleVal('Mute', True)
 
 class SayPerks(Task.InputTask):
   '''
@@ -78,7 +82,8 @@ class SayPerks(Task.InputTask):
     self.stopNow()
     names = self.getPerkNames()
     n = len(names)
-    self.sayInfo(text=self.getAppName(), template=str(n)+' perks in %s.')
+    # i18n: %d is number of perks, %s is application name
+    self.sayInfo(text=(n, self.getAppName()), template=_('%d perks in %s.'))
     self.sayInfo(text=', '.join(names))
 
 class ReloadPerks(Task.InputTask):
@@ -92,7 +97,8 @@ class ReloadPerks(Task.InputTask):
     Reloads L{Perk}s then makes an announcement.
     '''
     self.stopNow()
-    self.sayInfo(text=self.getAppName(), template='reloaded perks in %s')
+    # i18n: %s is application name
+    self.sayInfo(text=self.getAppName(), template=_('reloaded perks in %s'))
     self.reloadPerks()
 
 class ShowHideMonitors(Task.InputTask):
