@@ -6,7 +6,7 @@ derive.
 @author: Pete Brunet
 @author: Larry Weiss
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
@@ -15,7 +15,7 @@ this distribution, and is available at
 U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import weakref
-import LSRConstants, LSRInterfaces
+import AEConstants, LSRInterfaces
 from i18n import _
 
 class TaskTools(object):
@@ -41,8 +41,7 @@ class TaskTools(object):
   @type tier: L{Tier}
   @ivar perk: The L{Perk} owning the L{Task} subclass
   @type perk: L{Perk}
-  @ivar layer: Layer on which the event occurred, one of L{Task.LAYER_FOCUS},
-    L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+  @ivar layer: Layer on which the event occurred
   @type layer: integer
   @ivar lsr_state: The state object shared across all L{Tier}s
   @type lsr_state: L{AEState.AEState}
@@ -52,14 +51,10 @@ class TaskTools(object):
   @ivar task_por:  The L{POR} for the Task or Perk subclass, set by 
     L{preExecute}
   @type task_por: L{POR}
-  @ivar timestamp: Time at which an event occurred. Not guaranteed to be 
-    delivered for all events nor to be based on the same absolute zero across
-    all devices.
-  @type timestamp: float
   @ivar tools_initialized: Is the instance ready to be executed?
   @type tools_initialized: boolean
-  @ivar def_out: Default output device to use for all L{say} calls in this
-    L{Perk} module.
+  @ivar def_out: Default output device to use for all L{Task.Tools.Output.say}
+    calls in this L{Perk} module.
   @type def_out: weakref.proxy to L{AEOutput}
   '''
   def __init__(self):
@@ -78,7 +73,6 @@ class TaskTools(object):
     self.tier = None
     self.layer = None
     self.acc_eng = None
-    self.timestamp = 0
     self.def_out = None
     self.tools_initialized = False
     
@@ -131,6 +125,9 @@ class TaskTools(object):
     self.perk = None
     self.acc_eng.unloanManagers(self)
     self.acc_eng = None
+    # important! throw away this weak reference else the callback keeps this
+    # object alive
+    self.def_out = None
     self.tools_initialized = False
 
   def close(self):
@@ -139,18 +136,15 @@ class TaskTools(object):
     '''
     pass
   
-  def preExecute(self, layer, event_por=None, timestamp=0):
+  def preExecute(self, layer, event_por=None):
     '''
     Configures this L{TaskTools} object before execution.
 
-    @param layer: Layer on which the event occurred, one of 
-      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     @param event_por: Point of regard received with the event that triggered
       this execution
     @type event_por: L{POR}
-    @param timestamp: Time at which the event occurred
-    @type timestamp: float
     @return: Is the instance ready for execution?
     @rtype: boolean
     '''
@@ -162,9 +156,8 @@ class TaskTools(object):
     self.lsr_state = self.tier_man.getState()
     self.perk_state = self.perk.getState()
 
-    # store the layer and timestamp
+    # store the layer
     self.layer = layer
-    self.timestamp = timestamp
 
     # use the POR from the event or the last saved Task POR if there is no POR 
     # associated with the event
@@ -202,11 +195,11 @@ class TaskTools(object):
     @param ref: Weak proxy that is no longer valid
     @type ref: weakref.proxy
     '''
+    if not self.tools_initialized:
+      return
     # get an output device to serve as the default output
     caps = self.perk.getIdealOutput()
     dev = self.device_man.getOutputByCaps(caps)
-    if dev is None:
-      dev = self.device_man.getFirstOutput()
     try:
       self.def_out = weakref.proxy(dev, self._changeDefaultOutput)
     except TypeError:
