@@ -4,7 +4,7 @@ manging L{Tier}s, L{Perk}s, and L{Task}s.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
@@ -12,32 +12,16 @@ available under the terms of the BSD license which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/bsd-license.php}
 '''
-import weakref
+import weakref, sys
 from Error import *
 import AEEvent
 import Base
-import LSRConstants, UIRegistrar
+import AEConstants, UIRegistrar
 
 class System(Base.TaskTools):
   '''
   Provides methods for managing L{Tier}s, L{Perk}s, and L{Task}s.
-  
-  @cvar LOADED: Constant indicating L{UIElement}s loaded
-  @type LOADED: integer
-  @cvar INSTALLED: Constant indicating L{UIElement}s installed
-  @type INSTALLED: integer
-  @cvar TIER_ASSOCIATED: Constant indicating L{Perk}s associated with the 
-    current L{Tier} in the active profile
-  @type TIER_ASSOCIATED: integer
-  @cvar ALL_ASSOCIATED: Constant indicating L{UIElement}s associated with the 
-    active profile
-  @type ALL_ASSOCIATED: integer
   '''
-  LOADED = 0
-  INSTALLED = 1
-  TIER_ASSOCIATED = 2
-  ALL_ASSOCIATED = 3
-  
   def getProfileName(self):
     '''
     Gets the name of the profile LSR is running under.
@@ -128,18 +112,18 @@ class System(Base.TaskTools):
     @param kind: Kind of L{UIElement} to fetch metadata for, one of 
       L{UIRegistrar.ALL_KINDS}
     @type kind: string
-    @param which: Which names to get: L{INSTALLED}, L{TIER_ASSOCIATED}, 
-      or L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: All descriptions
     @rtype: list of string
     '''
     reg = UIRegistrar
-    if which == self.INSTALLED:
+    if which == AEConstants.UIE_INSTALLED:
       names = reg.listInstalled(kind)
-    elif which == self.ALL_ASSOCIATED:
+    elif which == AEConstants.UIE_ALL_ASSOCIATED:
       names = reg.listAssociated(kind, self.acc_eng.getProfile())
-    elif which == self.TIER_ASSOCIATED and kind == reg.PERK:
+    elif which == AEConstants.UIE_TIER_ASSOCIATED and kind == reg.PERK:
       names = reg.listAssociated(kind, self.acc_eng.getProfile(),
                              self.tier.getName())
     # make sure the UIE is loadable before listing it
@@ -153,18 +137,19 @@ class System(Base.TaskTools):
     @param kind: Kind of L{UIElement} to fetch metadata for, one of 
       L{UIRegistrar.ALL_KINDS}
     @type kind: string
-    @param which: Which names to get: L{INSTALLED}, L{TIER_ASSOCIATED}, 
-      or L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: All descriptions
     @rtype: list of string
     '''
     reg = UIRegistrar
-    if which == self.INSTALLED:
+    if which == AEConstants.UIE_INSTALLED:
       return [uie.getName() for uie in (reg.loadOne(name) for name in 
                                         reg.listInstalled(kind))
                                         if uie is not None]
-    elif which in (self.ALL_ASSOCIATED, self.TIER_ASSOCIATED):
+    elif which in (AEConstants.UIE_ALL_ASSOCIATED, 
+                   AEConstants.UIE_TIER_ASSOCIATED):
       names = self._getClassNames(kind, which)
       if names is None:
         return None
@@ -179,18 +164,19 @@ class System(Base.TaskTools):
     @param kind: Kind of L{UIElement} to fetch metadata for, one of 
       L{UIRegistrar.ALL_KINDS}
     @type kind: string
-    @param which: Which descriptions to get: L{INSTALLED}, L{TIER_ASSOCIATED}, 
-      or L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: All descriptions
     @rtype: list of string
     '''
     reg = UIRegistrar
-    if which == self.INSTALLED:
+    if which == AEConstants.UIE_INSTALLED:
       return [uie.getDescription() for uie in (reg.loadOne(name) for name in 
                                                reg.listInstalled(kind))
                                                if uie is not None]
-    elif which in (self.ALL_ASSOCIATED, self.TIER_ASSOCIATED):
+    elif which in (AEConstants.UIE_ALL_ASSOCIATED, 
+                   AEConstants.UIE_TIER_ASSOCIATED):
       names = self._getClassNames(kind, which)
       if names is None:
         return None
@@ -198,7 +184,7 @@ class System(Base.TaskTools):
                                                for name in names)
                                                if uie is not None]
 
-  def getPerkMetadata(self, which=LOADED):
+  def getPerkMetadata(self, which=AEConstants.UIE_LOADED):
     '''
     Gets metadata for all L{Perk}s in this L{Tier}, all installed L{Perk}s,
     L{Perk}s associated with this L{Tier}, or all L{Perk}s associated with this
@@ -209,8 +195,8 @@ class System(Base.TaskTools):
     
     @todo: PP: optimize
     
-    @param which: Which names to get: L{LOADED}, L{INSTALLED}, 
-      L{TIER_ASSOCIATED}, or L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Class names, names, descriptions
     @rtype: list of 3-tuple of string
@@ -219,7 +205,7 @@ class System(Base.TaskTools):
                self.getPerkNames(which),
                self.getPerkDescriptions(which))
 
-  def getDeviceMetadata(self, which=LOADED):
+  def getDeviceMetadata(self, which=AEConstants.UIE_LOADED):
     '''    
     Gets metadata for all L{AEOutput} and L{AEInput} devices currently loaded
     in, installed in, or associated with the active profile depending on the
@@ -229,8 +215,8 @@ class System(Base.TaskTools):
     
     @todo: PP: optimize
     
-    @param which: Which names to get: L{LOADED}, L{INSTALLED}, or
-      L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Class names, names, descriptions
     @rtype: list of 3-tuple of string
@@ -239,7 +225,7 @@ class System(Base.TaskTools):
                self.getDeviceNames(which),
                self.getDeviceDescriptions(which))
 
-  def getMonitorMetadata(self, which=LOADED):
+  def getMonitorMetadata(self, which=AEConstants.UIE_LOADED):
     '''    
     Gets metadata for all L{AEMonitor}s currently loaded in, installed in, or
     associated with the active profile depending on the which flag.
@@ -248,8 +234,8 @@ class System(Base.TaskTools):
     
     @todo: PP: optimize
     
-    @param which: Which names to get: L{LOADED}, L{INSTALLED}, or
-      L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Class names, names, descriptions
     @rtype: list of 3-tuple of string
@@ -258,7 +244,7 @@ class System(Base.TaskTools):
                self.getMonitorNames(which),
                self.getMonitorDescriptions(which))
   
-  def getChooserMetadata(self, which=ALL_ASSOCIATED):
+  def getChooserMetadata(self, which=AEConstants.UIE_ALL_ASSOCIATED):
     '''        
     Gets metadata for all L{AEChooser}s installed in or associated with the
     active profile depending on the which flag.
@@ -268,8 +254,8 @@ class System(Base.TaskTools):
     @todo: PP: optimize
     
     @note: Getting loaded chooser names is not currently supported.   
-    @param which: Which names to get: L{LOADED}, L{INSTALLED}, or
-      L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Class names, names, descriptions
     @rtype: list of 3-tuple of string
@@ -278,7 +264,7 @@ class System(Base.TaskTools):
                self.getChooserNames(which),
                self.getChooserDescriptions(which))
 
-  def getDeviceClassNames(self, which=LOADED):
+  def getDeviceClassNames(self, which=AEConstants.UIE_LOADED):
     '''
     Gets the programmatic class names of all L{AEOutput} and L{AEInput} devices
     currently loaded in, installed in, or associated with the active profile
@@ -286,18 +272,18 @@ class System(Base.TaskTools):
     
     The default value is used if which is invalid.
     
-    @param which: Which names to get: L{LOADED}, L{INSTALLED}, or 
-      L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Names of all L{AEOutput} and L{AEInput} classes
     @rtype: list of string
     '''
-    if which != self.LOADED:
+    if which != AEConstants.UIE_LOADED:
       return self._getClassNames(UIRegistrar.DEVICE, which)
     else:
       return [dev.getClassName() for dev in self.device_man.getDevices()]
     
-  def getDeviceNames(self, which=LOADED):
+  def getDeviceNames(self, which=AEConstants.UIE_LOADED):
     '''    
     Gets the human readable names of all L{AEOutput} and L{AEInput} devices
     currently loaded in, installed in, or associated with the active profile
@@ -305,18 +291,18 @@ class System(Base.TaskTools):
     
     The default value is used if which is invalid.
     
-    @param which: Which names to get: L{LOADED}, L{INSTALLED}, or
-      L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Names of all L{AEOutput} and L{AEInput} devices
     @rtype: list of string
     '''
-    if which != self.LOADED:
+    if which != AEConstants.UIE_LOADED:
       return self._getNames(UIRegistrar.DEVICE, which)
     else:
       return [dev.getName() for dev in self.device_man.getDevices()]
     
-  def getDeviceDescriptions(self, which=LOADED):
+  def getDeviceDescriptions(self, which=AEConstants.UIE_LOADED):
     '''    
     Gets the descriptions of all L{AEOutput} and L{AEInput} devices
     currently loaded in, installed in, or associated with the active profile
@@ -324,18 +310,18 @@ class System(Base.TaskTools):
     
     The default value is used if which is invalid.
     
-    @param which: Which descriptions to get: L{LOADED}, L{INSTALLED}, or
-      L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Names of all L{AEOutput} and L{AEInput} devices
     @rtype: list of string
     '''
-    if which != self.LOADED:
+    if which != AEConstants.UIE_LOADED:
       return self._getDescriptions(UIRegistrar.DEVICE, which)
     else:
       return [dev.getDescription() for dev in self.device_man.getDevices()]
   
-  def getMonitorClassNames(self, which=LOADED):
+  def getMonitorClassNames(self, which=AEConstants.UIE_LOADED):
     '''    
     Gets the programmatic class names of all L{AEMonitor}s currently loaded in,
     installed in, or associated with the active profile depending on the which 
@@ -343,13 +329,13 @@ class System(Base.TaskTools):
     
     The default value is used if which is invalid.
     
-    @param which: Which names to get: L{LOADED}, L{INSTALLED}, or 
-      L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Names of all L{AEMonitor} classes
     @rtype: list of string
     '''
-    if which != self.LOADED:
+    if which != AEConstants.UIE_LOADED:
       return self._getClassNames(UIRegistrar.MONITOR, which)
     else:
       text = []
@@ -361,7 +347,7 @@ class System(Base.TaskTools):
           text.append(mon.getClassNames())
       return text
     
-  def getMonitorNames(self, which=LOADED):
+  def getMonitorNames(self, which=AEConstants.UIE_LOADED):
     '''    
     Gets the human readable names of all L{AEMonitor}s currently loaded in,
     installed in, or associated with the active profile depending on the which 
@@ -369,13 +355,13 @@ class System(Base.TaskTools):
     
     The default value is used if which is invalid.
     
-    @param which: Which names to get: L{LOADED}, L{INSTALLED}, or
-      L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Names of all L{AEMonitor}s
     @rtype: list of string
     '''
-    if which != self.LOADED:
+    if which != AEConstants.UIE_LOADED:
       return self._getNames(UIRegistrar.MONITOR, which)
     else:
       text = []
@@ -387,7 +373,7 @@ class System(Base.TaskTools):
           text.append(mon.getNames())
       return text
     
-  def getMonitorDescriptions(self, which=LOADED):
+  def getMonitorDescriptions(self, which=AEConstants.UIE_LOADED):
     '''    
     Gets the human readable descriptions of all L{AEMonitor}s currently loaded
     in, installed in, or associated with the active profile depending on the 
@@ -395,13 +381,13 @@ class System(Base.TaskTools):
     
     The default value is used if which is invalid.
     
-    @param which: Which descriptions to get: L{LOADED}, L{INSTALLED}, or
-      L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Descriptions of all L{AEMonitor}s
     @rtype: list of string
     '''
-    if which != self.LOADED:
+    if which != AEConstants.UIE_LOADED:
       return self._getDescriptions(UIRegistrar.MONITOR, which)
     else:
       text = []
@@ -413,7 +399,7 @@ class System(Base.TaskTools):
           text.append(mon.getDescription())
       return text
   
-  def getChooserClassNames(self, which=ALL_ASSOCIATED):
+  def getChooserClassNames(self, which=AEConstants.UIE_ALL_ASSOCIATED):
     '''    
     Gets the programmatic class names of all L{AEChooser}s installed in or
     associated with the active profile depending on the which flag.
@@ -421,15 +407,16 @@ class System(Base.TaskTools):
     The default value is used if which is invalid.
     
     @note: Getting loaded chooser names is not currently supported.
-    @param which: Which names to get: L{INSTALLED} or L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Names of all L{AEChooser} classes
     @rtype: list of string
     '''
-    if which != self.LOADED:
+    if which != AEConstants.UIE_LOADED:
       return self._getClassNames(UIRegistrar.CHOOSER, which)
     
-  def getChooserNames(self, which=ALL_ASSOCIATED):
+  def getChooserNames(self, which=AEConstants.UIE_ALL_ASSOCIATED):
     '''    
     Gets the human readable names of all L{AEChooser}s installed in or
     associated with the active profile depending on the which flag.
@@ -437,15 +424,16 @@ class System(Base.TaskTools):
     The default value is used if which is invalid.
     
     @note: Getting loaded chooser names is not currently supported.
-    @param which: Which names to get: L{INSTALLED} or L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Names of all L{AEChooser}s
     @rtype: list of string
     '''
-    if which != self.LOADED:
+    if which != AEConstants.UIE_LOADED:
       return self._getNames(UIRegistrar.CHOOSER, which)
     
-  def getChooserDescriptions(self, which=ALL_ASSOCIATED):
+  def getChooserDescriptions(self, which=AEConstants.UIE_ALL_ASSOCIATED):
     '''
     Gets the descriptions of all L{AEChooser}s installed in or associated with
     the active profile depending on the which flag.
@@ -453,15 +441,16 @@ class System(Base.TaskTools):
     The default value is used if which is invalid.
     
     @note: Getting loaded chooser names is not currently supported.
-    @param which: Which descriptions to get: L{INSTALLED} or L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Descriptions of all L{AEChooser}s
     @rtype: list of string
     '''
-    if which != self.LOADED:
+    if which != AEConstants.UIE_LOADED:
       return self._getDescriptions(UIRegistrar.CHOOSER, which)
 
-  def getPerkClassNames(self, which=LOADED):
+  def getPerkClassNames(self, which=AEConstants.UIE_LOADED):
     '''
     Gets the programmatic class names of all L{Perk}s in this L{Tier}, all
     installed L{Perk}s, L{Perk}s associated with this L{Tier}, or all L{Perk}s 
@@ -469,18 +458,18 @@ class System(Base.TaskTools):
     
     The default value is used if which is invalid.
     
-    @param which: Which names to get: L{LOADED}, L{INSTALLED}, 
-      L{TIER_ASSOCIATED}, or L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Names of all L{Perk} classes
     @rtype: list of string
     '''
-    if which != self.LOADED:
+    if which != AEConstants.UIE_LOADED:
       return self._getClassNames(UIRegistrar.PERK, which)
     else:
       return [perk.getClassName() for perk in self.tier.getPerks()]
 
-  def getPerkNames(self, which=LOADED):
+  def getPerkNames(self, which=AEConstants.UIE_LOADED):
     '''    
     Gets the human readable names of all L{Perk}s in this L{Tier}, all
     installed L{Perk}s, L{Perk}s associated with this L{Tier}, or all L{Perk}s 
@@ -488,18 +477,18 @@ class System(Base.TaskTools):
     
     The default value is used if which is invalid.
     
-    @param which: Which names to get: L{LOADED}, L{INSTALLED}, 
-      L{TIER_ASSOCIATED}, or L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Names of all L{Perk}s
     @rtype: list of string
     '''
-    if which != self.LOADED:
+    if which != AEConstants.UIE_LOADED:
       return self._getNames(UIRegistrar.PERK, which)
     else:
       return [perk.getName() for perk in self.tier.getPerks()]
   
-  def getPerkDescriptions(self, which=LOADED):
+  def getPerkDescriptions(self, which=AEConstants.UIE_LOADED):
     '''    
     Gets the human readable descriptions of all L{Perk}s in this L{Tier}, all
     installed L{Perk}s, L{Perk}s associated with this L{Tier}, or all L{Perk}s 
@@ -507,16 +496,16 @@ class System(Base.TaskTools):
     
     The default value is used if which is invalid.
     
-    @param which: Which descriptions to get: L{LOADED}, L{INSTALLED}, 
-      L{TIER_ASSOCIATED}, or L{ALL_ASSOCIATED}
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
     @type which: integer
     @return: Descriptions of all L{Perk}s
     @rtype: list of string
     '''
-    if which != self.LOADED:
+    if which != AEConstants.UIE_LOADED:
       return self._getDescriptions(UIRegistrar.PERK, which)
     else:
-      return [perk.getDescription() for perk in self.tier.getPerks()]    
+      return [perk.getDescription() for perk in self.tier.getPerks()]
     
   def registerTask(self, task):
     '''
@@ -576,20 +565,59 @@ class System(Base.TaskTools):
     try:
       # then try as a timer task
       # make sure it's a timer task first
-      task.getInterval
+      task.getTimerInterval
       perk.unregisterTimerTask(task)
+      count += 1  
     except AttributeError:
       pass
     try:
       # now try as a named task
       perk.unregisterNamedTask(task.getIdentity())
       count += 1
-    except (AttributeError, ValueError):
+    except (AttributeError, ValueError, KeyError):
       pass
     if not count:
       # not any kind of known task
       raise NotImplementedError
     
+  def chainTask(self, name, link, target):
+    '''
+    Chains the L{Task} with the given name to the target L{Task}. If
+    link is L{AEConstants.CHAIN_BEFORE}, the linked L{Task} will execute before
+    the target. If link is L{AEConstants.CHAIN_AFTER}, the linked L{Task} will
+    execute after. If link is L{AEConstants.CHAIN_AROUND}, the linked L{Task}
+    will execute in lieu of the target, but can execute the target manually
+    using L{doTask}.
+    
+    @param target: Name of the L{Task} to link to
+    @type target: string
+    @param link: One of the CHAIN constants in L{AEConstants.Tools} 
+    @type link: integer
+    @param name: Name of the L{Task} to link
+    @type name: string
+    '''
+    self.perk.linkTaskToChain(name, link, target)
+
+  def unchainTask(self, name, link, target):
+    '''
+    Unchains the L{Task} with the given name to the target L{Task}. The link
+    parameter has the same meaning as in L{chainTask}. If link is set to None,
+    one reference to the named L{Task} is removed from the before, after, and
+    around chains.
+    
+    @note: The target L{Task} must be registered at the time of invocation.
+    @param target: Name of the L{Task} to unlink from
+    @type target: string
+    @param link One of the CHAIN constants in L{AEConstants.Tools} or None
+    @type link: integer
+    @param name: Name of the L{Task} to unlink
+    @type name: string
+    @raise ValueError: When the named L{Task} is not unlinked at least once 
+    @raise KeyError: When the target L{Task} does not have a chain segement in
+      this L{Perk} for the given link type
+    '''
+    self.perk.unlinkTaskFromChain(name, link, target)
+       
   def getTierNamedTask(self, name):
     '''
     Gets the L{Task} with the given name if it is registered anywhere in the
@@ -616,8 +644,7 @@ class System(Base.TaskTools):
     
     @param event_type: Desired type of L{AEEvent}
     @type event_type: L{AEEvent} class
-    @param task_layer: Layer on which the desired L{Task}s are registered, one 
-      of L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param task_layer: Layer on which the desired L{Task}s are registered
     @type task_layer: integer
     @return: List of all L{Task}s that handle the given event type and on 
       the given layer in the L{Tier} which owns this L{Perk}
@@ -646,8 +673,7 @@ class System(Base.TaskTools):
     
     @param event_type: Desired type of L{AEEvent}
     @type event_type: L{AEEvent} class
-    @param task_layer: Layer on which the desired L{Task}s are registered, one 
-      of L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param task_layer: Layer on which the desired L{Task}s are registered
     @type task_layer: integer
     @return: List of all L{Task}s that handle the given event type on the given
       layer in this L{Perk}
@@ -792,11 +818,11 @@ class System(Base.TaskTools):
       chooser(self.event_man, self.tier.getIdentity())
     except ValueError, ex:
       # singleton already exists, reactivate
-      ex.args[0].activate(timestamp=self.timestamp, **kwargs)
+      ex.args[0].activate(**kwargs)
       return None
     else:
       # call init to let the chooser initialize itself
-      chooser.init(timestamp=self.timestamp, **kwargs)
+      chooser.init(**kwargs)
       # register to handle events from this chooser
       self.perk.registerChooserTask(chooser, self)
       return chooser
@@ -812,7 +838,8 @@ class System(Base.TaskTools):
     # unregister the task observing this chooser
     self.perk.unregisterChooserTask(chooser)
 
-  def doTask(self, task_name, **kwargs):
+  def doTask(self, task_name, chain=True, propagate=True, layer=None,
+             **kwargs):
     '''
     Immediately executes a L{Task} registered under the given name in this
     L{Tier}. Keyword arguments for L{Task} execution may be provided. The 
@@ -820,14 +847,32 @@ class System(Base.TaskTools):
     executed and this L{Task}'s L{task_por} is updated with the value of the
     pointer after the other L{Task} has finished execution.
     
+    The chain parameter dictates whether only the named L{Task} is executed
+    (False), or if all other L{Task}s chained to it are as well (True). The
+    propagate parameter determines whether the execute or update method is
+    called on the L{Task}. This param is not typically specified by a L{Task}
+    or L{Perk}. It is used internally by L{Task.CyclicTask} to handle the
+    propagation flag in Task chains.
+    
     @param task_name: Name of the L{Task} to execute
     @type task_name: string
+    @param chain: Execute all L{Task}s chained to the one named or just the
+      one named?
+    @type chain: boolean
+    @param propagate: Should chained L{Task}s be executed (True) or updated 
+      (False)? 
+    @type propagate: boolean
+    @param layer: Event layer on which the L{Task} will execute. Defaults to 
+      the same layer as the caller if None.
+    @type layer: integer
     @param kwargs: Arbitrary keyword arguments to pass to the L{Task}, some of
       which may be required, others which may be optional, and others which
       may be completely ignored
     @type kwargs: dictionary
     '''
-    self.tier.manageNamedTask(self.task_por, task_name, self.layer, kwargs)
+    if layer is None: layer = self.layer
+    self.tier.manageNamedTask(self.task_por, task_name, layer, kwargs,
+                              chain=chain)
     self.task_por = self.getPointerPOR()
     
   def blockNTasks(self, n, task_type, condition=lambda **x: True):
@@ -949,22 +994,53 @@ class System(Base.TaskTools):
     self.device_man.unloadDevices()
     self.device_man.loadDevices()
     
-  def signalSetting(self, state, name, value, **kwargs):
+  def setTempVal(self, name, value):
     '''
-    Sends an L{AEEvent.SettinChange} indicating the setting with the given name
-    in the given L{AEState} object changed its value. Any keyword arguments
-    are provided as-is to observers of the event. The source L{Perk} of the 
-    change is included in the event.
+    Stores the given value under the given name. The data will be available via
+    L{getTempVal} to all L{Task}s executing or updating in response to the 
+    current L{AEEvent}. This includes cascading L{Task.EventTask}s, chained 
+    L{Task.InputTask}s with names, etc. Once the event has been handled by all
+    L{Task}s, all stored information is discarded automatically.
     
-    @param state: State object having the setting that changed
-    @type state: L{AEState}
-    @param name: Name of the setting variable that changed
-    @type name: string
-    @param value: New value of the setting
+    The name must be an immutable object. The value can be an arbitrary object.
+    Only the value from the most recent call with a given name is stored.
+    
+    @param name: Name to associate with the value
+    @type name: immutable
+    @param value: Any value to store
     @type value: object
-    @param kwargs: Arbitrary data to be delivered with the event
-    @type kwargs: dictionary
     '''
-    event = AEEvent.SettingChange(self.tier.getIdentity(), self.perk, state, 
-                                  name, value, **kwargs)
-    self.event_man.postEvents(event)
\ No newline at end of file
+    self.tier.setTempVal(name, value)
+
+  def getTempVal(self, name):
+    '''
+    Gets the value previously stored under the given name using L{setTempVal}.
+    See that method for details.
+    
+    If no value has been stored under the given name, None is returned.
+
+    @param name: Name of the stored data
+    @type name: immutable
+    @return: Value stored under the given name or None
+    @rtype: object
+    '''
+    try:
+      return self.tier.getTempVal(name)
+    except KeyError:
+      return None
+
+  def registerConstants(self, scope, *names):
+    '''
+    Adds new constants to L{AEConstants} such that they are visible to other
+    L{Perk}s. If any of the named constants do not exist, an exception will
+    be raised. If any of the named constants are already registered, they will
+    be silently ignored by L{AEConstants.register}.
+    
+    @param scope: Scope in which the constants are defined (e.g. globals())
+    @type scope: dictionary
+    @param names: Names of constants to register
+    @type names: list of string
+    @raise KeyError: When one of the named constants does not exist in the 
+      scope
+    '''
+    AEConstants.register(dict(((name, scope[name]) for name in names)))
\ No newline at end of file
