@@ -3,16 +3,16 @@ Defines a L{Task} to execute in response to a L{AEEvent.ViewChange}.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
-import EventTask, Constants
+import Base, EventTask, AEConstants
 import AEEvent
 
 class ViewTask(EventTask.EventTask):
@@ -20,24 +20,11 @@ class ViewTask(EventTask.EventTask):
   Executed when a view change occurs.
   
   This class registers its name and whether it should be monitored by default
-  in an L{AEMonitor} using the L{Constants.registerTaskType} function when this
+  in an L{AEMonitor} using the L{Base.registerTaskType} function when this
   module is first imported. An L{AEMonitor} can use this information to build
-  its menus.
-  
-  @cvar GAINED: View was gained by activating it
-  @type GAINED: integer
-  @cvar LOST: View was lost by deactivating it
-  @type LOST: integer
-  @cvar STARTUP: View was identified at LSR startup
-  @type STARTUP: integer
-  @cvar FIRST_GAINED: View was gained by activating it during LSR startup
-  @type FIRST_GAINED: integer
+  its menus. 
   '''
-  LOST = AEEvent.ViewChange.LOST
-  GAINED = AEEvent.ViewChange.GAINED
-  STARTUP = AEEvent.ViewChange.STARTUP
-  FIRST_GAINED = AEEvent.ViewChange.FIRST_GAINED
-  Constants.registerTaskType('ViewTask', True)
+  Base.registerTaskType('ViewTask', True)
   
   def getEventType(self):
     '''
@@ -55,10 +42,9 @@ class ViewTask(EventTask.EventTask):
     @type por: L{POR}
     @param title: Title of the view
     @type title: string
-    @param gained: One of L{LOST}, L{GAINED}, L{STARTUP}, L{FIRST_GAINED}
+    @param gained: The kind of view change event
     @type gained: integer
-    @param layer: Layer on which the event occurred, one of
-      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     '''
     pass
@@ -72,21 +58,20 @@ class ViewTask(EventTask.EventTask):
     @type por: L{POR}
     @param title: Title of the view
     @type title: string
-    @param gained: One of L{LOST}, L{GAINED}, L{STARTUP}, L{FIRST_GAINED}
+    @param gained: The kind of view change event
     @type gained: integer
-    @param layer: Layer on which the event occurred, one of
-      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     @return: True to allow other L{Task}s to process this event
     @rtype: boolean
     '''
-    if gained == self.GAINED:
+    if gained == AEConstants.EVENT_VIEW_GAINED:
       return self.executeGained(por=por, title=title, layer=layer, **kwargs)
-    elif gained == self.LOST:
+    elif gained == AEConstants.EVENT_VIEW_LOST:
       return self.executeLost(por=por, title=title, layer=layer, **kwargs)
-    elif gained == self.STARTUP:
+    elif gained == AEConstants.EVENT_VIEW_STARTUP:
       return self.executeStartup(por=por, title=title, layer=layer, **kwargs)
-    elif gained == self.FIRST_GAINED:
+    elif gained == AEConstants.EVENT_VIEW_FIRST_GAINED:
       return self.executeFirstGained(por=por, title=title, layer=layer, 
                                      **kwargs)
     
@@ -99,8 +84,7 @@ class ViewTask(EventTask.EventTask):
     @type por: L{POR}
     @param title: Title of the view
     @type title: string
-    @param layer: Layer on which the event occurred, one of
-      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     @return: True to allow other L{Task}s to process this event
     @rtype: boolean
@@ -116,8 +100,7 @@ class ViewTask(EventTask.EventTask):
     @type por: L{POR}
     @param title: Title of the view
     @type title: string
-    @param layer: Layer on which the event occurred, one of
-      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     @return: True to allow other L{Task}s to process this event
     @rtype: boolean
@@ -133,8 +116,7 @@ class ViewTask(EventTask.EventTask):
     @type por: L{POR}
     @param title: Title of the view
     @type title: string
-    @param layer: Layer on which the event occurred, one of
-      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     @return: True to allow other L{Task}s to process this event
     @rtype: boolean
@@ -150,8 +132,7 @@ class ViewTask(EventTask.EventTask):
     @type por: L{POR}
     @param title: Title of the view
     @type title: string
-    @param layer: Layer on which the event occurred, one of
-      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     @return: True to allow other L{Task}s to process this event
     @rtype: boolean
