@@ -8,14 +8,11 @@ Defines a class responsible for handling events in the currently focused view.
   L{ViewManager} by default
 @type DEFAULT_EVENTS: list
 
-@todo: PP: the AT-SPI specific code for walking the desktop should be moved to
-  an adapter 
-
 @author: Peter Parente
 @author: Pete Brunet
 @author: Brett Clippingdale
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
@@ -37,17 +34,24 @@ CHECK_INTERVAL = 60000
 
 # events that we should always watch, regardless of Task interests
 DEFAULT_EVENTS = [ViewChange, FocusChange, SelectorChange, CaretChange, 
-                  PropertyChange]
+                  PropertyChange, PrivateChange]
 
 class ViewManager(object):
   '''
   Stores a root accessible representing the active view when an event occurs
-  indicating that the view may have changed.
+  indicating that the view may have changed. Two view roots must be tracked
+  by this object. The first, the raw view, is needed by L{Adapters} which will
+  handle raw view change events. The second, the L{AcessEngine} view, is needed
+  by the rest of the system when processing L{AEEvent}s.
   
   @ivar event_manager: Reference to the L{EventManager}
   @type event_manager: L{EventManager}
-  @ivar active_view: Root L{POR} representing the active view
-  @type active_view: L{POR}
+  @ivar raw_view: Root accessible representing the active view according to the
+    raw event stream.
+  @type raw_view: L{pyLinAcc.Accessible}
+  @ivar ae_view: Root L{POR} represnting the active view according to the 
+    L{AEEvent} stream.
+  @type ae_view: L{POR}
   @ivar tier_manager: Reference to the L{TierManager} for clearing dead L{Tier}
     objects on a set interval
   @type tier_manager: L{TierManager}
@@ -55,8 +59,8 @@ class ViewManager(object):
     L{Task} in any L{Perk} in all L{Tier}s. Used to optimize event dispatch and
     processing. Unwanted events are ignored.
   @type wanted_events: dictionary
-  @ivar activated: Is there an activated view somewhere?
-  @type activated: boolean
+  @ivar raw_active: Is there an raw active view?
+  @type raw_active: boolean
   '''
   def __init__(self, ae):
     '''
@@ -68,9 +72,9 @@ class ViewManager(object):
     '''
     self.tier_manager = None
     self.event_manager = None
-    self.active_view = None
+    self.raw_view = None
     self.wanted_events = {}
-    self.activated = False
+    self.raw_active = False
     # register for a callback to check living applications
     ae.addTimer(self.onIterateApps, CHECK_INTERVAL)
       
@@ -108,7 +112,7 @@ class ViewManager(object):
     that application, creates a L{Tier} for the application and loads all of
     its L{Perk}s. If the L{Tier} is created, this method sends
     L{AEEvent.ViewChange} messages to the L{Tier} with a flag of
-    L{AEEvent.ViewChange.ViewChange.STARTUP} such that L{Perk}s in the L{Tier}
+    L{AEConstants.EVENT_VIEW_STARTUP} such that L{Perk}s in the L{Tier}
     may initialize themselves and begin any desired background processes.
     '''
     d = pyLinAcc.Registry.getDesktop(0)
@@ -127,9 +131,9 @@ class ViewManager(object):
       name = aai.getAccAppName()
       # inform the tier manager
       if self.tier_manager.createTier(name, aid, app, True) is not None:
-        events.append(ViewChange(app, ViewChange.STARTUP))
+        events.append(ViewChange(app, AEConstants.EVENT_VIEW_STARTUP))
       # if we've found the active app, just continue
-      if self.active_view is not None:
+      if self.raw_view is not None:
         continue
       aan = IAccessibleNav(app)
       # iterate over all top level windows in the app
@@ -141,11 +145,13 @@ class ViewManager(object):
         # look for one that has state active
         wai = IAccessibleInfo(win)
         if wai.hasAccState('active'):
-          self.active_view = win
+          self.raw_view = win
+          self.raw_active = True
           break
     # send a view change gained event to the active tier
-    if self.active_view is not None:
-      events.append(ViewChange(self.active_view, ViewChange.FIRST_GAINED))
+    if self.raw_view is not None:
+      events.append(ViewChange(self.raw_view, 
+                               AEConstants.EVENT_VIEW_FIRST_GAINED))
     self.event_manager.postEvents(*events)
       
   def setEventInterest(self, kind, wants):
@@ -175,52 +181,79 @@ class ViewManager(object):
                                    *IEventHandler().getRawEvents(kind))
     self.wanted_events[kind] = count
     
-  def getActivated(self):
+  def getRawActive(self):
     '''
     Gets the whether some view is active or not.
     
-    @return: Value of L{activated}
+    @return: Value of L{raw_active}
     @rtype: boolean
     '''
-    return self.activated
+    return self.raw_active
   
-  def setActivated(self, val):
+  def setRawActive(self, val):
     '''
     Sets the whether some view is active or not.
     
-    @param val: Value to store in L{activated}
+    @param val: Value to store in L{raw_active}
     @type val: boolean
     '''
-    self.activated = val
+    self.raw_active = val
 
-  def getView(self):
+  def getRawView(self):
     '''
-    Gets the root accessible of the active view.
+    Gets the root L{POR} of the active view according to the raw event
+    stream. This is the view known to L{Adapter}s.
     
-    @return: Root L{POR} of the active view
+    @see: L{setRawView}
+    @return: Root L{POR} of the raw active view
     @rtype: L{POR}
     '''
-    return self.active_view
+    return self.raw_view
   
-  def setView(self, accessible):
+  def setRawView(self, accessible):
     '''    
-    Stores the root accessible corresponding to the received event to
-    represent the active view.
+    Stores the root L{POR} of the active view according to the raw event 
+    stream. This is the view known to L{Adapter}s.
     
+    @see: L{getRawView}
     @param accessible: Event source that triggered the change of view. Usually
       a top level window or panel.
     @type accessible: L{pyLinAcc.Accessible}
     @return: Was a new view root set or not?
     @rtype: boolean
     '''
+    rv = False
     # walk to the first POR
-    root = POR(accessible) #AccessibleWalker(POR(accessible)).getFirstPOR()
+    root = POR(accessible)
     # see if it is different from our current root POR
-    if root != self.active_view:
+    if root != self.raw_view:
       # store the new view
-      self.active_view = root
-      return True
-    return False
+      self.raw_view = root
+      rv = True
+    return rv
+  
+  def getAEView(self):
+    '''
+    Gets the root L{POR} of the active view according to the L{AEEvent} stream.
+    This is the view known to L{Perk}s.
+    
+    @see: L{setAEView}
+    @return: Root L{POR} of the active L{AccessEngine} view
+    @rtype: L{POR}
+    '''
+    return self.ae_view
+  
+  def setAEView(self, por):
+    '''
+    Stores the root L{POR} of the active view according to the L{AEEvent 
+    stream. This is the view known to L{Adapter}s.
+    
+    @see: L{getAEView}
+    @param por: New root L{POR} of the active L{AccessEngine} view. May be
+      set to None if there is no active view.
+    @type por: L{POR}
+    '''
+    self.ae_view = por
   
   def onIterateApps(self):
     '''
@@ -257,18 +290,18 @@ class ViewManager(object):
     @type event: L{pyLinAcc.Event.Event}
     '''
     # only forward events if there is a view
-    if self.active_view is not None:
+    if self.raw_view is not None:
       eh = IEventHandler(event.source)
       eh.getAEEvents(event, self.event_manager.postEvents)
     
   def onRawViewEvent(self, event):
-    '''    
+    '''   
     Creates a L{AEEvent.ViewChange} event in response to a window activation or
     deactivation. Also responds to create or destroy events from floating
-    windows. Called in response to any of the raw events listed in
-    L{VIEW_EVENTS} and registered with the L{EventManager} in the
-    L{ViewManager.init} method when this object was created. All events in
-    L{VIEW_EVENTS} are monitored as indicating view changes.
+    windows. Called in response to any of the raw events returned by
+    L{LSRInterfaces.IEventHandler.getAEViewEvents} and registered with the
+    L{EventManager} in the L{ViewManager.init} method when this object was
+    created.
 
     @param event: Event that may indicate the view has changed
     @type event: L{pyLinAcc.Event.Event}
