@@ -10,45 +10,25 @@ processing (at least for English text).
 
 @var VOWELS: Vowels in the used to determine if a word can be spoken
 @type VOWELS: string
-@var WORD_NON_BLANK: Definition indicating all non-blank characters should be 
-  considered the main part of a word
-@type WORD_NON_BLANK: integer
-@var WORD_ALPHABETIC: Definition indicating all letters should be considered 
-  the main part of a word
-@type WORD_ALPHABETIC: integer
-@var WORD_ALPHA_NUMERIC: Definition indicating all letters and numbers should 
-  be considered the main part of a word
-@type WORD_ALPHA_NUMERIC: integer
-@var WORD_ALPHA_PUNCT: Definition indicating all letters and punctuation should
-  be considered the main part of a word
-@type WORD_ALPHA_PUNCT: integer
-@var WORD_ALPHA_NUMERIC_PUNCT: Definition indicating all letters, numbers, and
-  punctuation should be considered the main part of a word
-@type WORD_ALPHA_NUMERIC_PUNCT: integer
 
 @author: Peter Parente
 @author: Larry Weiss
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import unicodedata as ud
 import AEState
 from POR import POR
+from AEConstants import WORD_NON_BLANK, WORD_ALPHABETIC, WORD_ALPHA_NUMERIC, \
+     WORD_ALPHA_PUNCT, WORD_ALPHA_NUMERIC_PUNCT, WORD_LAST
 from i18n import _
 
-# constants for State.WordDef
-WORD_NON_BLANK = 0
-WORD_ALPHABETIC = 1
-WORD_ALPHA_NUMERIC = 2
-WORD_ALPHA_PUNCT = 3
-WORD_ALPHA_NUMERIC_PUNCT = 4
-
 # define vowels if there are any in a language
 VOWELS = _('AEIOUYaeiouy')
 
@@ -57,30 +37,66 @@ class WordState(AEState.AEState):
   Settings for L{Word} parsing. This class contains the set of all settings 
   that will be respected by the parser. Subclasses may override these settings
 
-  @ivar Caps: When set to True, capitalization is preserved when presenting
-    text. Defaults to True.
-  @type Caps: boolean
-  @ivar MaxRepeat: Specifies the minimum number of times a character must be
-    found in sequence before it is considered a repeat. Defaults to 4.
-  @type MaxRepeat: integer
-  @ivar WordDef: Set to NON_BLANK to define the main part of a word to only
-    include non-blank characters. Set to ALPHABETIC to define the main part 
-    of a word to only include alphabetic characters. Set to ALPHA_NUMERIC to
-    define the main part of a word to only include alphabetic and numeric
-    characters. Set to ALPHA_NUMERIC_PUNCT to define the main part of a word
-    to only include characters that are alphabetic, numeric, or punctuation.
-    Defaults to NON_BLANK.
-  @type WordDef: integer
-  @ivar Ignore: Set to a strings of characters that should be treated as if 
-    they were blank. Defaults to an empty string.
-  @type Ignore: string
+  The following variables are not truly instance variables, but are proxied
+  by L{AEState.Setting} objects.
+
+  Caps (bool): When set to True, capitalization is preserved when presenting
+  text. Defaults to True.
+ 
+  MaxRepeat (integer): Specifies the minimum number of times a character must 
+  be found in sequence before it is considered a repeat. Defaults to 4.
+
+  WordDef (enum): Set to NON_BLANK to define the main part of a word to only
+  include non-blank characters. Set to ALPHABETIC to define the main part 
+  of a word to only include alphabetic characters. Set to ALPHA_NUMERIC to
+  define the main part of a word to only include alphabetic and numeric
+  characters. Set to ALPHA_NUMERIC_PUNCT to define the main part of a word
+  to only include characters that are alphabetic, numeric, or punctuation.
+  Defaults to NON_BLANK.
+
+    Ignore (string): Set to a string of characters that should be treated as if 
+  they were blank. Defaults to an empty string.
+  '''
+  def __init__(self):
+    '''
+    Initializes all settings objects.
+    '''
+    super(WordState, self).__init__()
+    self._initWordSettings()
+      
+  def _initWordSettings(self):
+    '''
+    Creates all settings for L{Word} parsing.
+    '''
+    self.newBool('Caps', True, _('Preserve caps?'), 
+                 _('When set, capitalization is preserved in the output. '
+                   'Otherwise, all strings are lowercased.'))
+    self.newNumeric('MaxRepeat', 4, _('Maximum repeat'), 1, 100, 0, 
+                    _('Defines the maximum number of times a '
+                      'character should be allowed to repeat without the word '
+                      'being spelled.'))
+    self.newEnum('WordDef', WORD_NON_BLANK, _('Word definition'), 
+                 {_('Non-blank') : WORD_NON_BLANK,
+                  _('Alphanumeric and punct.') : WORD_ALPHA_NUMERIC_PUNCT,
+                  _('Alphanumeric') : WORD_ALPHA_NUMERIC,
+                  _('Alpha and punct.') : WORD_ALPHA_PUNCT,
+                  _('Alphabetic') : WORD_ALPHABETIC},
+                 _('Defines the characters that comprise a word.'))
+    self.newString('Ignore', '', _('Ignored characters'), 
+                   _('Defines the characters to be treated as blanks.'))
+
+class DefaultWordState(object):
+  '''
+  Dummy L{WordState} look-alike used by L{getContextFromString} as a default 
+  set of settings when no state object is specified. Defines all the same
+  attributes as L{WordState}, but without instantiating unecessary settings.
   '''
   Caps = True
   MaxRepeat = 4
   WordDef = WORD_NON_BLANK
   Ignore = ''
 
-def getContextFromString(string, por, state=WordState()):
+def getContextFromString(string, por, state=DefaultWordState):
   '''  
   Gets the previous, current, and next L{Word}s relative to the given L{POR}. 
   If any word is missing, a None value is returned in its place. The string is 
@@ -115,8 +131,8 @@ def getContextFromString(string, por, state=WordState()):
   # handle the case when the por is in the last word
   return words[i-1], words[i], None
 
-def buildWordsFromString(string, por=None, state=WordState(), main_ob=None,
-                         trail_ob = None):
+def buildWordsFromString(string, por=None, state=DefaultWordState, 
+                         main_ob=None, trail_ob = None):
   '''
   Parses the given string to build a list of L{Word}s using the given state and
   the given L{POR}. When no L{POR} is given a dummy POR is constructed. Each
@@ -180,7 +196,7 @@ class Word(object):
   L{WordState} to improve its presentation to the user via a speech or other
   output device while the trailing part remains unprocessed. The value of
   WordDef determines what characters lie in the main and trailing parts of each
-  word. The following constants are available in L{LSRConstants}.
+  word. The following constants are available in L{AEConstants}.
   
     - WORD_NON_BLANK: All non-blank characters are added to the main part
     - WORD_ALPHABETIC: All characters considered letters in the current locale
@@ -405,8 +421,8 @@ class Word(object):
 
   def isVowel(self, ch):
     '''
-    Determines if the given character is a vowel. Currently only checks for
-    vowels in the Latin alphabet.
+    Determines if the given character is a vowel. Relies on a translator to
+    list all vowels in the current locale.
 
     @param ch: Character to test
     @type ch: string
@@ -426,6 +442,34 @@ class Word(object):
     '''      
     return ch.isupper()
   
+  def getCharValue(self, ch):
+    '''
+    Gets the unicode hex value for a character sans the 0x prefix.
+    
+    @param ch: Single character
+    @type ch: string
+    @return: Hex value of the character
+    @rtype: string
+    '''
+    return hex(ord(ch))[2:]
+  
+  def getCharName(self, ch):
+    '''
+    Gets the unicode name of the character, one of the strings listed in the
+    U{http://unicode.org/charts/charindex.html}. If the character could not be
+    determined from the given string, returns an empty string. Note that these 
+    names are not localized.
+    
+    @param ch: Single character
+    @type ch: string
+    @return: Name of the character
+    @rtype: string
+    '''
+    try:
+      return ud.name(unicode(ch)).lower()
+    except Exception:
+      return ''
+  
   def getCharDescription(self, ch):
     '''
     Gets a localized description of the given character. The most detailed
@@ -496,9 +540,9 @@ class Word(object):
     return self.more
   
   def hasRepeat(self):
-    '''
-    Gets if this L{Word} has a character repeated more than 
-    L{WordState.MaxRepeat} times or not.
+    '''    
+    Gets if this L{Word} has a character repeated more than the maximum number
+    of repetitions allowed or not.
     
     @return: Does this L{Word} containg a repeated character?
     @rtype: boolean
@@ -682,8 +726,7 @@ class Word(object):
   #import sys
   #def printWords(words):
     #for w in words:
-      #sys.stdout.write(str(w))
-      #print
+      #sys.stdout.write('"%s"' % str(w))
     #for w in words:
       #print w.getPOR()
   
@@ -691,10 +734,10 @@ class Word(object):
     #def __init__(self, **kwargs):
       #self.__dict__.update(kwargs)
 
-  #test = "JAWS is a screen reader. I'd like $10.00 for free. Hey--look at that!"
-  #state = TestState(WordDef=WORD_NON_BLANK, Caps=True, CapExpand=False, 
+  #test = "JAWS is a screen reader. I'd like to say, 'I'd like $10.00 for free.' Hey--look at that!"
+  #state = TestState(WordDef=WORD_NON_BLANK, Caps=True, CapExpand=True, 
                     #NumExpand=True, Ignore='', MaxRepeat=3)
-  #printWords(buildWordsFromString(test, state))
+  #printWords(buildWordsFromString(test, None, state))
   #for i in range(len(test)):
     #p, c, n = getContextFromString(test, state, POR(None, None, i))
     #try: print p.getPOR(), p,
